/**** <lalVerbatim file="InitSearchHV"> ***********
Author: Julien Sylvestre
$Id: InitSearch.h,v 1.3 2003/09/04 21:10:41 julien Exp $ 
***** </lalVerbatim> ***********************************/

#ifndef _INITSEARCH_H
#define _INITSEARCH_H

#include <LALWrapperInterface.h>
#include <lal/LALDatatypes.h>
#include <lal/LIGOMetadataTables.h>
#include <lal/LALRCSID.h>
#include <lal/StdBurstSearch.h>
#include <lal/Comm.h>

#ifdef  __cplusplus
extern "C" {
#endif

NRCSID (INITSEARCHH, "$Id: InitSearch.h,v 1.3 2003/09/04 21:10:41 julien Exp $");

  /*
#define DEBUGBURST 
  */

#ifdef linux
#undef WORDS_BIGENDIAN
#endif

#define MAXNBURSTS 50000
#define MAXBLOB 800000

#define GW_STRAIN_DATA "GW_STRAIN_DATA"
#define GW_STRAIN_PSD "GW_STRAIN_PSD"

#define RCHECKSTATUSPTR( statusptr ) \
  if ( (statusptr)->statusPtr->statusCode ) \
  { \
    SETSTATUS( statusptr, -1, "Recursive error" ); \
    DETATCHSTATUSPTR(status); \
    RETURN (status); \
  } else (void)(0)

#define RASSERT(p1,p2,p3,p4) if(!(p1)) {ABORT(p2,p3,p4);}

  typedef struct tagBurstParameterList {
    
    struct tagBurstParameterList *next;

    BurstParameter param;

  } BurstParameterList;

  typedef enum {TFCLUSTERS,SLOPE,POWER} BurstETG;

typedef struct tagBurstSearchParams {

  CHAR channel[LIGOMETA_CHANNEL_MAX];

  UINT4 Ndata;

  BurstParameter waveforms;

  UINT4 Nwaveforms;

  REAL4TimeVectorSeries *wave_pc;

  BurstParameter waveform_amplitudes;

  UINT4 Nwaveform_amplitude;

  BurstParameter alpha;

  UINT4 Nalpha;

  BurstParameter delta;

  UINT4 Ndelta;

  BurstParameter psi;

  UINT4 Npsi;

  UINT4 MAXinj;

  UINT4 Ninj;

  BurstParameter injTimes;

  UINT4 Ninj_per_segment;

  BurstETG ETG;

  BurstParameterList ETGparams;

  BOOLEAN ETGparamsLOCK;

  UINT4 Nparams;

  UINT4 *NNparams;

  BurstOutputDataSegment data;

  BurstOutputParameters oparams;

  BOOLEAN searchMaster;

  void (*ETGfun)(LALStatus *, EventIDColumn *, REAL4TimeVectorSeries *, BurstParameter *);

  INT4 outputMethod;

  BOOLEAN noLALBurstOutput;

  CHAR dsoOutput[1024];

} BurstSearchParams;


typedef enum
{
  ExchParamList,
  ExchFinished,
  ExchDeadSlave,
  ExchFileLock
}
ExchObjectType;


  void ParseParameter( 
		      LALStatus *stat,
		      BurstParameter *params,
		      CHAR *argv,
		      UINT4 *N
		      );

  void BuildOutput(LALStatus *status,
		   LALSearchOutput *output,
		   EventIDColumn     *burstEventList,
		   CHAR               *tag,
		   UINT4 t0Sec,
		   UINT4 t0NanoSec,
		   REAL8 tDuration,
		   UINT4 numSlaves,
		   UINT4 zerobid
		   );
  
  void BuildOutputBinary(LALStatus *status,
			 LALSearchOutput *output,
			 EventIDColumn     *burstEventList,
			 CHAR               *tag,
			 UINT4 t0Sec,
			 UINT4 t0NanoSec,
			 UINT4 zerobid
			 );

  void BuildOutputFileBinary(LALStatus *status,
			     CHAR *file,
			     EventIDColumn     *burstEventList,
			     CHAR               *tag,
			     UINT4 t0Sec,
			     UINT4 t0NanoSec,
			     InitExchParams *initExchParams
			     );

/******** <lalErrTable file="TFCInitSearchHErrTab"> ********/
#define INITSEARCHH_ENULL     1
#define INITSEARCHH_ENNUL     2
#define INITSEARCHH_EALOC     3
#define INITSEARCHH_EARGS     4
#define INITSEARCHH_ESPOS      5
#define INITSEARCHH_ENAN      6
#define INITSEARCHH_EAOR      7
#define INITSEARCHH_EPOS      8
#define INITSEARCHH_EMMF      9
#define INITSEARCHH_ENAM      10
#define INITSEARCHH_ETAB      11
#define INITSEARCHH_EIN      12
#define INITSEARCHH_ERANK     13
#define INITSEARCHH_EMAXBLOB 14
#define INITSEARCHH_EETG 15
#define INITSEARCHH_EETGDATA 16
#define INITSEARCHH_ENB 17
#define INITSEARCHH_EUI 18
#define INITSEARCHH_EFILE 19

#define INITSEARCHH_MSGENULL     "Null pointer"
#define INITSEARCHH_MSGENNUL     "Non-null pointer"
#define INITSEARCHH_MSGEALOC     "Memory allocation error"
#define INITSEARCHH_MSGEARGS     "Wrong number of arguments"
#define INITSEARCHH_MSGESPOS      "Parameter <= 0 expected to be strictly positive"
#define INITSEARCHH_MSGENAN       "Numerical parameter is NaN"
#define INITSEARCHH_MSGEAOR      "Parameter expected to be in [0,1] is out of range"
#define INITSEARCHH_MSGEPOS      "Parameter < 0 when expected to be positive or zero"
#define INITSEARCHH_MSGEMMF      "Parameter minf > maxf"
#define INITSEARCHH_MSGENAM       "Channel/IFO name is zero length"
#define INITSEARCHH_MSGETAB       "Unknown table name"
#define INITSEARCHH_MSGEIN       "invalid input"
#define INITSEARCHH_MSGERANK     "rank of node"
#define INITSEARCHH_MSGEMAXBLOB "Maximum allowed BLOB size (1 Mo) exceeded"
#define INITSEARCHH_MSGEETG "ETG function generated a SIGSEGV or SIGBUS signal!"
#define INITSEARCHH_MSGEETGDATA "ETG function modifies its input!"
#define INITSEARCHH_MSGENB "Maximum number of burst exceeded!"
#define INITSEARCHH_MSGEUI "Unimplemented feature!"
#define INITSEARCHH_MSGEFILE "File error!"

/******** </lalErrTable> ********/


#ifdef  __cplusplus
}
#endif

#endif /* _INITSEARCH_H  */


