/********************************** <lalVerbatim file="TFCFinalizeSearchCV">
Author: Julien Sylvestre
$Id: FinalizeSearch.c,v 1.2 2003/09/04 21:10:41 julien Exp $
**************************************************** </lalVerbatim> */

#include <lal/LALStdlib.h>
#include <InitSearch.h>
#include <lal/AVFactories.h>
#include <lal/SeqFactories.h>

NRCSID( FINALIZESEARCHC, "tfclusters $Id: FinalizeSearch.c,v 1.2 2003/09/04 21:10:41 julien Exp $" );

void
LALFinalizeSearch(
    LALStatus             *status,
    void                 **searchParams
    )
{
  
  BurstSearchParams *params;
  BurstParameter *ptr, *ptr2;
  BurstParameterList *pl, *pl2;
  UINT4 i;

  INITSTATUS (status, "LALFinalizeSearch", FINALIZESEARCHC);
  ATTATCHSTATUSPTR (status);

#ifdef DEBUGBURST
      fprintf(stderr,"ENTERING FINALIZESEARCH...\n");
      fflush(NULL);
#endif

  if ( !(*searchParams) ){
    ABORT( status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL );
  }
  params = (BurstSearchParams *) *searchParams;

  #define CLEAR_BURSTPARAMETER while(ptr) { \
    if(ptr->char_) { \
      LALFree(ptr->char_); \
    } \
    if(ptr->int4_) { \
      LALFree(ptr->int4_); \
    } \
    if(ptr->real4_) { \
      LALFree(ptr->real4_); \
    } \
    if(ptr->real4vector_) { \
      if(ptr->real4vector_->data) { \
        LALFree(ptr->real4vector_->data); \
      } \
    } \
    ptr2 = ptr->next; \
    LALFree(ptr); \
    ptr = ptr2; \
  }

  ptr = params->waveforms.next;
  CLEAR_BURSTPARAMETER

  ptr = params->alpha.next;
  CLEAR_BURSTPARAMETER

  ptr = params->delta.next;
  CLEAR_BURSTPARAMETER

  ptr = params->psi.next;
  CLEAR_BURSTPARAMETER

  pl = params->ETGparams.next;

  if(params->ETGparamsLOCK) {
    BurstParameter *bptr;

    bptr = pl->param.next;
    while (bptr) {
      memcpy(&ptr, bptr->char_, sizeof(BurstParameter *));
      CLEAR_BURSTPARAMETER

      free(bptr->char_);
      ptr = bptr->next;
      free(bptr);
      bptr = ptr;
    }
    
    free(pl);

  } else {
    while(pl) {
      ptr = pl->param.next;
      CLEAR_BURSTPARAMETER

	pl2 = pl->next;
      LALFree(pl);
      pl = pl2;
    }
  }

#undef CLEAR_BURSTPARAMETER

#ifdef DEBUGBURST
      fprintf(stderr,"FINALIZESEARCH: done with burstparameters\n");
      fflush(NULL);
#endif

  if(params->wave_pc) {

#ifdef DEBUGBURST
      fprintf(stderr,"FINALIZESEARCH: need to delete wave_pc: %u\n",params->Nwaveforms);
      fflush(NULL);
#endif

    for(i=0; i<params->Nwaveforms; i++) {

#ifdef DEBUGBURST
      fprintf(stderr,"FINALIZESEARCH: loop: %u\n",i);
      fflush(NULL);
#endif

      LALFree(params->wave_pc[i].data->data);

#ifdef DEBUGBURST
      fprintf(stderr,"FINALIZESEARCH: loop + 1: %u\n",i);
      fflush(NULL);
#endif

      LALFree(params->wave_pc[i].data);
    }

#ifdef DEBUGBURST
      fprintf(stderr,"FINALIZESEARCH: loop done: %u\n",i);
      fflush(NULL);
#endif

    LALFree(params->wave_pc);

#ifdef DEBUGBURST
      fprintf(stderr,"FINALIZESEARCH: done: %u\n",i);
      fflush(NULL);
#endif

  }

#ifdef DEBUGBURST
      fprintf(stderr,"FINALIZESEARCH: done with injected waveforms\n");
      fflush(NULL);
#endif

  if(params->NNparams) {

#ifdef DEBUGBURST
      fprintf(stderr,"FINALIZESEARCH: need to delete NNparams\n");
      fflush(NULL);
#endif

    LALFree(params->NNparams);
  }

#ifdef DEBUGBURST
      fprintf(stderr,"FINALIZESEARCH: done with NNparams\n");
      fflush(NULL);
#endif

  if(params->data.data) {
    LALDestroyVectorSequence(status->statusPtr, &(params->data.data->data));
    CHECKSTATUSPTR (status);
    LALFree(params->data.data);
  }

#ifdef DEBUGBURST
      fprintf(stderr,"FINALIZESEARCH: done with data.data\n");
      fflush(NULL);
#endif

  if(params->data.spec) {
    LALDestroyVector(status->statusPtr, &(params->data.spec->data));
    CHECKSTATUSPTR (status);
    LALFree(params->data.spec);
  }

#ifdef DEBUGBURST
      fprintf(stderr,"FINALIZESEARCH: done with data.spec\n");
      fflush(NULL);
#endif

  if(params->data.resp) {
    LALCDestroyVector(status->statusPtr, &(params->data.resp->data));
    CHECKSTATUSPTR (status);
    LALFree(params->data.resp);
  }

#ifdef DEBUGBURST
      fprintf(stderr,"FINALIZESEARCH: done with data.resp\n");
      fflush(NULL);
#endif

  if(params->data.preprocessing_filter) {
    LALDestroyREAL4IIRFilter(status->statusPtr, &(params->data.preprocessing_filter));
    CHECKSTATUSPTR (status);
    LALFree(params->data.preprocessing_filter);
  }

#ifdef DEBUGBURST
      fprintf(stderr,"FINALIZESEARCH: done with data.preprocessing\n");
      fflush(NULL);
#endif

  LALFree(*searchParams);
  *searchParams = NULL;

#ifdef DEBUGBURST
  fprintf(stderr,"EXITING FINALIZESEARCH...\n");
  fflush(NULL);
#endif

  DETATCHSTATUSPTR (status);
  RETURN (status);
  
}
