/* -*- mode: c; c-basic-offset: 2; -*- */
/******** <lalVerbatim file="ExtTrigFinalizeSearchCV"> ********
Author: Rahkola, R
$Id: ExtTrigFinalizeSearch.c,v 1.12 2003/10/07 19:44:03 jolien Exp $
********* </lalVerbatim> ********/

/* Include input/output functions if we're in the debug stage */
#ifdef DEBUG_EXTTRIG
#include <stdio.h>
#endif

/* standard C functions */
#include <math.h>

/* standard LAL functions*/
#include <lal/LALStdlib.h>
#include <lal/AVFactories.h>

/* LAL MPI interface */
#include <lal/Comm.h>

/* Associated headers */
#include <ExtTrig.h>
#include <ExtTrigFinalizeSearch.h>

#ifdef __cplusplus
extern "C" {
#endif

NRCSID( EXTTRIGFINALIZESEARCHC, "$Id: ExtTrigFinalizeSearch.c,v 1.12 2003/10/07 19:44:03 jolien Exp $" );

/************************* wrapperAPI function definitions  ******************/

/*
 * Function LALFinalizeSearch
 *
 * Finalize routine does nothing but check arguments and free memory
 */
void
LALFinalizeSearch(
    LALStatus             *status,
    void                 **searchParams)
{
  ETSearchParams *const params = (ETSearchParams *) *searchParams;
  UINT4           i; /* iterator(s) */    

  INITSTATUS( status, "LALFinalizeSearch", EXTTRIGFINALIZESEARCHC );
  ATTATCHSTATUSPTR (status);

#ifdef DEBUG_EXTTRIG
  fprintf( stdout, "exttrig:LALFinalizeSearch - %s\n", params->id );
  fflush ( stdout );
#endif

  /*
   *
   * clean up
   *
   */
  /* free memory used by ETInfoParams */
  LALFree( params->info );
  params->info = NULL;


  /**********************
   **********************
   *** Free memory used by ETResult
   *** ALL: free xCorrSpectra, if used
   *** MASTER ONLY: free xCorrLag, xCorrDelay, xCorrValue, variance1, variance2
   *** ALL: free ETResult
   **********************
   **********************/
  if (params->initParams->initMask & 48) {
    for (i=0; i < params->result->filled; i++) {
      LALCDestroyVector( status->statusPtr,
			 &(params->result->xCorrSpectra[i]->data) );
      CHECKSTATUSPTR( status );
      LALFree( params->result->xCorrSpectra[i] );
    }
    LALFree( params->result->xCorrSpectra );
  }

  if ( params->searchMaster ) {
    LALFree( params->result->xCorrLag );
    LALFree( params->result->xCorrDelay );
    LALFree( params->result->xCorrValue );
    LALFree( params->result->variance1 );
    LALFree( params->result->variance2 );
  }

  LALFree( params->result );
  params->result = NULL;

  /*  free memory used by ETSegList */
  LALDestroyETSegList( status->statusPtr, &(params->segments) );
  CHECKSTATUSPTR( status );

  /* free memory used by ETDataSegmentVector */
  LALDestroyETDataSegmentVector( status->statusPtr, &(params->data) );
  CHECKSTATUSPTR( status );

  /* free memory used by the FFT Plan (used to be SZeroPadAndFFTParameters) */
/*    LALDestroyVector( status->statusPtr, &(params->fftParams->window) ); */
/*    CHECKSTATUSPTR( status ); */
  TRY( LALDestroyRealFFTPlan(status->statusPtr, &(params->fftParams)), status);
/*    LALFree( params->fftParams ); */
  params->fftParams = NULL;

  /* free memory used by the COMPLEX8FrequencySeries */
  LALCDestroyVector( status->statusPtr, &(params->fsNotch->data) );
  CHECKSTATUSPTR( status );
  LALFree( params->fsNotch );
  LALCDestroyVector( status->statusPtr, &(params->fsKernel->data) );
  CHECKSTATUSPTR( status );
  LALFree( params->fsKernel );
  params->fsKernel = params->fsNotch =  NULL;

  /* free memory used by the REAL4TimeSeries */
  LALSDestroyVector( status->statusPtr, &(params->tsChan2->data) );
  CHECKSTATUSPTR( status );
  LALFree( params->tsChan2 );
  LALSDestroyVector( status->statusPtr, &(params->tsChan1->data) );
  CHECKSTATUSPTR( status );
  LALFree( params->tsChan1 );
  params->tsChan1 = params->tsChan2 = NULL;

  /* free memory used by ETInitParams */
  LALFree( params->initParams );
  params->initParams = NULL;

  /* free ETSearchParams structure */
  LALFree( *searchParams );
  *searchParams = NULL;

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

#ifdef __cplusplus
}
#endif
