/*----------------------------------------------------------------------- 
 * 
 * File Name: InspiralEngine.h
 *
 * Author: Brown, D. A.
 * 
 * Revision: $Id: InspiralEngine.h,v 1.3 2003/07/22 23:48:52 duncan Exp $
 * 
 *-----------------------------------------------------------------------
 */

#if 0
<lalVerbatim file="InspiralEngineHV">
Author: Brown, D. A.
$Id: InspiralEngine.h,v 1.3 2003/07/22 23:48:52 duncan Exp $
</lalVerbatim> 

<lalLaTeX>
\section{Header \texttt{InspiralEngine.h}}
\label{s:InspiralEngine.h}

Provides functions to drive the filtering functions in the package
\texttt{findchirp}. Contains the functions necessary to execute a flat
or heirarchical inspiral search using the LDAS \texttt{wrapperAPI} or
the LALapps program \texttt{findchirp}.

\subsection*{Synopsis}

\begin{verbatim}
#include <lal/InspiralEngine.h>
\end{verbatim}

\input{InspiralEngineHDoc}

</lalLaTeX>
#endif

#ifndef _INSPIRALENGINEH_H
#define _INSPIRALENGINEH_H

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <lal/LALStdlib.h>
#include <lal/Random.h>
#include <lal/DataBuffer.h>
#include <lal/LALInspiral.h>
#include <lal/LALInspiralBank.h>
#include <lal/FindChirp.h>
#include <lal/FindChirpSP.h>
#include <lal/FindChirpEngine.h>
#include <lal/GeneratePPNInspiral.h>
#include <lal/SimulateCoherentGW.h>
#include <lal/Units.h>
#include <lal/Inject.h>
#include <lal/Comm.h>
#include "InspiralExch.h"

#ifdef  __cplusplus
extern "C" {
#pragma }
#endif


NRCSID (INSPIRALENGINEHH, "$Id: InspiralEngine.h,v 1.3 2003/07/22 23:48:52 duncan Exp $");

#if 0
<lalLaTeX> 
\subsection*{Error codes} 
</lalLaTeX>
#endif
/* <lalErrTable> */
#define INSPIRALENGINEH_ENULL 1
#define INSPIRALENGINEH_ENNUL 2
#define INSPIRALENGINEH_ENUMZ 3
#define INSPIRALENGINEH_EDELT 4
#define INSPIRALENGINEH_ERHOZ 5
#define INSPIRALENGINEH_ECHIZ 6
#define INSPIRALENGINEH_ETMPL 7
#define INSPIRALENGINEH_EALOC 8
#define INSPIRALENGINEH_ERANK 9
#define INSPIRALENGINEH_EUEXT 10
#define INSPIRALENGINEH_ELVEL 11
#define INSPIRALENGINEH_ESEGZ 12
#define INSPIRALENGINEH_EUSIM 13
#define INSPIRALENGINEH_EWAVL 14
#define INSPIRALENGINEH_EHETR 15
#define INSPIRALENGINEH_ESPEC 16
#define INSPIRALENGINEH_ESTPE 17
#define INSPIRALENGINEH_EZERO 18
#define INSPIRALENGINEH_EAOVR 19
#define INSPIRALENGINEH_MSGENULL "Null pointer"
#define INSPIRALENGINEH_MSGENNUL "Non-null pointer"
#define INSPIRALENGINEH_MSGENUMZ "Data segment length is zero"
#define INSPIRALENGINEH_MSGEDELT "deltaT is zero or negative"
#define INSPIRALENGINEH_MSGERHOZ "snr squared threshold is zero or negative"
#define INSPIRALENGINEH_MSGECHIZ "chi squared threshold is zero or negative"
#define INSPIRALENGINEH_MSGETMPL "linked list of templates to filter in null"
#define INSPIRALENGINEH_MSGEALOC "Memory allocation error"
#define INSPIRALENGINEH_MSGERANK "Search node has incorrect rank"
#define INSPIRALENGINEH_MSGEUEXT "Unrecognised exchange type"
#define INSPIRALENGINEH_MSGELVEL "Invalid heriarchical template bank level"
#define INSPIRALENGINEH_MSGESEGZ "Number of data segments is zero"
#define INSPIRALENGINEH_MSGEUSIM "Unkown simulation type requested"
#define INSPIRALENGINEH_MSGEWAVL "Simulated waveform is longer than dataseg"
#define INSPIRALENGINEH_MSGEHETR "Attempting to simulate heterodyned GW"
#define INSPIRALENGINEH_MSGESPEC "Strain power spectral density missing"
#define INSPIRALENGINEH_MSGESTPE "Unknown power spectrum estimation method"
#define INSPIRALENGINEH_MSGEZERO "Zero length"
#define INSPIRALENGINEH_MSGEAOVR "Array overrun"
/* </lalErrTable> */


#if 0
<lalLaTeX>
\subsection*{Types}
</lalLaTeX>
#endif

/* --- enumeraion type for simulations ----------------------------------- */
/* <lalVerbatim file="InspiralEngineHInspiralSimulationType"> */
typedef enum
{
  fcNoSim,
  fcCreateRhosqVec,
  fcGaussianNoise,
  fcGaussianNoiseInject,
  fcRealDataInject,
  fcBankMinimalMatch
}
InspiralSimulationType;
/* </lalVerbatim> */
#if 0
<lalLaTeX>
\subsubsection*{Enumeration Type \texttt{InspiralSimulationType}}
\idx[Type]{InspiralSimulationType}

\input{InspiralEngineHInspiralSimulationType}

\noindent This enumaration type lists the possible types of simulation
that can be performed by the \texttt{InspiralEngine} functions.  The choices
are:

\begin{description}
\item[\texttt{fcNoSim}] A regular data analysis run; no simulation performed.

\item[\texttt{fcCreateRhosqVec}] Create a vector contain the statistic
$\rho^2(t_j)$ when filtering.

\item[\texttt{fcGaussianNoise}] Replace the input data in the
\texttt{DataSegmentVector} with Gaussian noise and a simulated LIGO I response
function.

\item[\texttt{fcGaussianNoiseInject}] Replace the input data in the
\texttt{DataSegmentVector} with Gaussian noise and a simulated LIGO I response
function. Inject random signals into the data as descibed by the
\texttt{InspiralSimulationParams}.

\item[\texttt{fcGaussianNoiseInject}] Inject random signals into the input
data as descibed by the \texttt{InspiralSimulationParams}.

\item[\texttt{fcBankMinimalMatch}] Perform a bank minimal match simulation
using the parameters in the \texttt{InspiralSimulationParams} structure.
\end{description}
</lalLaTeX>
#endif

typedef enum
{
  fcSpecDatacond,
  fcSpecMean,
  fcSpecMedian,
  fcSpecMeanBandPassed,
  fcSpecMedianBandPassed
}
InspiralSpecType;

typedef struct
tagInspiralSimulationParams
{
  InspiralSimulationType       simType;        /* type of simulation        */
  UINT4                         simCount;       /* number of simulations     */
  UINT4                         injectStartIdx; /* index of the first inj    */
  SimInspiralTable             *injectEvent;    /* events to inject          */
  SnglInspiralTable            *loudestEvent;   /* array of loudest events   */
  REAL4                        *signalNorm;     /* (s|s) for each dataSeg    */
  REAL4                         mMin;           /* minimum mass for binary   */
  REAL4                         mMax;           /* maximum mass for binary   */
  REAL4                         fLowInject;     /* waveform low freq cutoff  */
  REAL4                         gaussianVarsq;  /* variance squared of noise */
  RandomParams                 *randomParams;   /* random seed container     */
  REAL4TimeSeries              *chan;           /* pointer to the data chunk */
}
InspiralSimulationParams;

typedef struct
tagInspiralSlaveParams
{
  UINT4                         bandPassed;
  UINT4                         haveSpec;
  REAL4Sequence                 dataChannel;
  InspiralSpecType             specType;
  UINT4                         dataConditioned;
  UINT4                        *inspiralDebugFlagPtr;
  REAL4                        *rhosqThreshVec;
  REAL4                        *chisqThreshVec;
  FILE                         *tmpltBankFilePtr;
  FILE                         *eventFilePtr;
  FindChirpStandardCandle      *candlePtr;
  FindChirpSegmentVector       *fcSegVec;
  FindChirpSPDataParams        *dataParams;
  FindChirpSPTmpltParams       *tmpltParams;
  FindChirpFilterParams        *filterParams;
  FindChirpFilterInput         *filterInput;
  InspiralSimulationParams    *simParams;
  BOOLEAN                      *notFinished;
  BOOLEAN                       useMPI;
  void                         *mpiComm;
}
InspiralSlaveParams;

enum ExchObjectType
{
  ExchInspiralTemplate,
  ExchInspiralEvent,
  ExchNumTmpltsFiltered,
  ExchFinished
};

typedef struct
tagInspiralMasterParams
{
  UINT4                         numTmpltExch;
  UINT4                         numTmpltsTotal;
  UINT4                         numTmpltsToFilter;
  UINT4                         numTmpltsFiltered;
  UINT4                        *inspiralDebugFlagPtr;
  UINT4Vector                  *bankSentVec;
  MPI_Comm                     *mpiComm;
  UINT4                        *numSlaves;
  InspiralTemplate             *tmpltBankHead;
  InspiralTemplateNode         *currentTmpltNode;
  InspiralTemplateNode         *tmpltNodeHead;
}
InspiralMasterParams;

void
InspiralMaster (
    LALStatus                  *status, 
    SnglInspiralTable         **eventList,
    UINT4                      *numEvents,
    InspiralMasterParams      *params 
    );

void
InspiralSlave (
    LALStatus                  *status, 
    SnglInspiralTable         **outputEventHandle,
    DataSegmentVector          *dataSegVec,
    InspiralSlaveParams        *params 
    );

#ifdef  __cplusplus
}
#endif

#endif /* _INSPIRALENGINEH_H */
