/*----------------------------------------------------------------------- 
* 
* File Name: FinalizeSearch.c
*
* Author: Brown, D. A.
* 
* Revision: $Id: FinalizeSearch.c,v 1.22 2003/07/22 23:56:39 duncan Exp $
* 
*-----------------------------------------------------------------------
 */

#include "FinalizeSearch.h"

NRCSID( FINALIZESEARCHC, PKG "$Id: FinalizeSearch.c,v 1.22 2003/07/22 23:56:39 duncan Exp $" );

void
LALFinalizeSearch(
    LALStatus             *status,
    void                 **searchParams
    )
{
  InspiralTemplate             *currentTmplt;
  InspiralSearchParams         *params          = NULL;
  InspiralMasterParams         *masterParamsPtr = NULL;
  InspiralSlaveParams          *slaveParamsPtr  = NULL;

  INITSTATUS( status, "LALFinalizeSearch", FINALIZESEARCHC );
  ATTATCHSTATUSPTR( status );

  ASSERT( *searchParams, status,
      FINALIZESEARCHH_ENULL, FINALIZESEARCHH_MSGENULL );

  /* we basically run InitSearch backwards, freeing anything that       */
  /* was allocated. we also need to free the linked list of template    */
  /* nodes that was allocated in ConditionData() and appened to in      */
  /* ApplySearch(). on the master we need to free numSlaves which was   */
  /* allocated in ApplySearch()                                         */

  params = (InspiralSearchParams *) *searchParams;
  masterParamsPtr = &(params->masterParams);
  slaveParamsPtr  = &(params->slaveParams);


 /*
  *
  * free template bank and linked list of template nodes
  *
  */


  if ( params->searchMaster )
  {
    for ( currentTmplt = params->tmpltBankHead; currentTmplt; 
        currentTmplt = currentTmplt->next )
    {
      LALI4DestroyVector( status->statusPtr, &(currentTmplt->segmentIdVec) );
      CHECKSTATUSPTR( status );
    }
    LALFindChirpDestroyInspiralBank( status->statusPtr, 
        &(params->tmpltBankHead) );
    CHECKSTATUSPTR( status );

    while ( params->masterParams.tmpltNodeHead )
    {
      LALFindChirpDestroyTmpltNode( status->statusPtr, 
          &(masterParamsPtr->tmpltNodeHead) );
      CHECKSTATUSPTR( status );
    }

    LALFree( masterParamsPtr->numSlaves );

    /* free the bank sent vector, if it exists */
    if ( masterParamsPtr->bankSentVec )
    {
      LALU4DestroyVector( status->statusPtr, &(masterParamsPtr->bankSentVec) );
      CHECKSTATUSPTR( status );
    }
  }
  else /* searchSlave */
  {
    LALFinalizeDataSegmentVector( status->statusPtr, &(params->dataSegVec) );
    CHECKSTATUSPTR( status );
#if 0
    /* free the DataSegments in the DataSegmentVector */
    for ( i = 0; i < params->dataSegVec->length; ++i )
    {
      /* point to current segment */
      DataSegment *currentSegment = params->dataSegVec->data + i;
      LALFree( currentSegment->chan->data );
      LALFree( currentSegment->chan );
    }

    /* free the dataSegVec to hold the input data for FindChirpData */
    LALFree( params->dataSegVec->data );
    LALFree( params->dataSegVec );
#endif

    /* free the PSD memory if it has been allocated */
    if ( slaveParamsPtr->specType )
    {
      LALFree( params->spec.data->data );
    }

    /* free memory for the frequency sequences in the spec and resp */
    if ( params->responseType == newCalibration )
    {
      LALFree( params->resp.data->data );
    }
    LALFree( params->resp.data );
    LALFree( params->spec.data );

    /* free memory for the input data time series */
    LALFree( params->chan.data );

    LALFindChirpChisqVetoFinalize( status->statusPtr, 
        slaveParamsPtr->filterParams->chisqParams, 
        params->initParams.numChisqBins );
    CHECKSTATUSPTR( status );
    LALDestroyFindChirpInput( status->statusPtr, 
        &(slaveParamsPtr->filterInput) );
    CHECKSTATUSPTR( status );

    LALFindChirpFilterFinalize( status->statusPtr, 
        &(slaveParamsPtr->filterParams) );
    CHECKSTATUSPTR( status );

    LALFindChirpSPTemplateFinalize( status->statusPtr, 
        &(slaveParamsPtr->tmpltParams) );
    CHECKSTATUSPTR( status );

    LALFindChirpSPDataFinalize( status->statusPtr, 
        &(slaveParamsPtr->dataParams) );
    CHECKSTATUSPTR( status );

    LALDestroyFindChirpSegmentVector( status->statusPtr, 
        &(slaveParamsPtr->fcSegVec) );
    CHECKSTATUSPTR( status );

    LALFree( slaveParamsPtr->rhosqThreshVec );
    LALFree( slaveParamsPtr->chisqThreshVec );

    if ( slaveParamsPtr->simParams ) 
    {
      while ( slaveParamsPtr->simParams->injectEvent )
      {
        SimInspiralTable *thisEvent = slaveParamsPtr->simParams->injectEvent;
        slaveParamsPtr->simParams->injectEvent =
          slaveParamsPtr->simParams->injectEvent->next;
        LALFree( thisEvent );
      }
      LALDestroyRandomParams( status->statusPtr, 
          &(slaveParamsPtr->simParams->randomParams) );
      CHECKSTATUSPTR( status );
      LALFree( slaveParamsPtr->simParams );
    }
  }

  /* free searchParams */
  LALFree( *searchParams );
  *searchParams = NULL;

  /* normal exit */
  DETATCHSTATUSPTR( status );
  RETURN( status );
}

