/*----------------------------------------------------------------------- 
 * 
 * File Name: TmpltInput.c
 *
 * Author: Brown, D. A.
 * 
 * Revision: $Id: InitSearchTmplt.c,v 1.7 2003/07/23 16:16:41 duncan Exp $
 * 
 *-----------------------------------------------------------------------
 */

#if 0
<lalVerbatim file="InitSearchTmpltCV">
Author: Brown, D. A.
$Id: InitSearchTmplt.c,v 1.7 2003/07/23 16:16:41 duncan Exp $
</lalVerbatim>

<lalLaTeX>
\subsection{Module \texttt{InitSearchTmplt.c}}
\label{inspiral:InitSearchTmplt.c}
Parses the template parameters passed using \texttt{-filterparams}.

\subsubsection*{Prototypes}
\vspace{0.1in}
\input{InitSearchTmpltCP}

\subsubsection*{Description}

\subsubsection*{Algorithm}

None.

\subsubsection*{Uses}
\begin{verbatim}
LALCalloc()
LALI4CreateVector()
\end{verbatim}

\subsubsection*{Notes}

\vfill{\footnotesize\input{InitSearchTmpltCV}}
</lalLaTeX>
#endif

#include "InitSearch.h"

NRCSID( TMPLTINPUTC, PKG "$Id: InitSearchTmplt.c,v 1.7 2003/07/23 16:16:41 duncan Exp $" );

/* <lalVerbatim file="InitSearchTmpltCP"> */
void
InspiralTmpltsFromCmdLine (
    LALStatus                  *status,
    InspiralSearchParams       *params,
    CHAR                      **argv
    )
/* </lalVerbatim> */
{
  INT4                  start = 0;
  INT4                  characters = 0;
  INT4                  numTmpltString;
  INT4                  flag;
  UINT4                 tmpltIndex = 0;
  CHAR                 *tmpltString;
  CHAR                 *result; 
  CHAR                  buffer[256];
  REAL4                 m1, m2;
  InspiralTemplate     *here = NULL;

  INITSTATUS( status, "InspiralTmpltsFromCmdLine", TMPLTINPUTC );
  ATTATCHSTATUSPTR( status );


  /*
   *
   * check input and output
   *
   */


  ASSERT( params, status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL );
  ASSERT( argv, status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL );


  /*
   *
   * set up variables
   *
   */


  numTmpltString = atoi( argv[INSPARG_TPRM] );
  tmpltString = argv[INSPARG_TMPL];


  /*
   *
   * parse the list of templates
   *
   */


  /* find the first semicolon */
  result = strchr( tmpltString, '/' );

  /* if there is no semicolon there should only be one template     */
  /* if there is a semicolon there should be more than one template */
  if ( !result && numTmpltString != 1 ) 
  {
    ABORT( status, INITSEARCHH_ETSTR, INITSEARCHH_MSGETSTR );
  }
  else if ( result && numTmpltString == 1 ) 
  {
    ABORT( status, INITSEARCHH_ETSTR, INITSEARCHH_MSGETSTR );
  }

  /* how many chars to copy */
  characters = ( result ) ? 
    result - tmpltString + 1 - start : strlen( tmpltString ) - start + 1;

  /* copt the chars into the buffer */
  strncpy( buffer, tmpltString + start, characters - 1 );
  buffer[characters - 1] = '\0';

  /* increment the start counter */
  start += characters;

  /* parse the line of templates */
  flag = sscanf( buffer, "%f, %f\n", &m1, &m2 ) - 2;

  ASSERT( !flag, status, 
      INITSEARCHH_ENDAT, INITSEARCHH_MSGENDAT );

  /* allocate the head of the linked list */
  here = params->tmpltBankHead = (InspiralTemplate *)
    LALCalloc( 1, sizeof( InspiralTemplate ) );
  ASSERT( here, status, 
      INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC );

  /* set the template parameters */
  here->number = tmpltIndex++;
  here->mass1 = m1;
  here->mass2 = m2;
  here->totalMass = m1 + m2;
  here->mu = m1*m2/here->totalMass;
  here->eta = here->mu/here->totalMass;
  here->minMatch = params->bankMatch;

  LALI4CreateVector( status->statusPtr, 
      &(here->segmentIdVec), params->initParams.numSegments );
  CHECKSTATUSPTR( status );
  memset( here->segmentIdVec->data, 0, 
      here->segmentIdVec->length * sizeof(INT4) );

  /* continue with the rest of the string */
  while( start < strlen( tmpltString ) )
  {
    /* find the next semicolon */
    result = strchr( tmpltString + start, '/' );

    /* how many chars to copy */
    characters = ( result ) ? 
      result - tmpltString + 1 - start : strlen( tmpltString ) - start + 1;

    /* copt the chars into the buffer */
    strncpy( buffer, tmpltString + start, characters - 1 );
    buffer[characters - 1] = '\0';

    /* increment the start counter */
    start += characters;

    /* read in the masses and allocate the first template */
    flag = sscanf( buffer, "%f, %f\n", &m1, &m2 ) - 2;
    if ( !flag )
    {
      here->next = (InspiralTemplate *)
        LALCalloc( 1, sizeof( InspiralTemplate ) );
      here = here->next;
      ASSERT( here, status, 
          INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC );

      here->number = tmpltIndex++;
      here->mass1 = m1;
      here->mass2 = m2;
      here->totalMass = m1 + m2;
      here->mu = m1*m2/here->totalMass;
      here->eta = here->mu/here->totalMass;

      LALI4CreateVector( status->statusPtr, 
          &(here->segmentIdVec), params->initParams.numSegments );
      CHECKSTATUSPTR( status );
      memset( here->segmentIdVec->data, 0, 
          here->segmentIdVec->length * sizeof(INT4) );
    }
    else
    {
      ABORT( status, INITSEARCHH_ETSTR, INITSEARCHH_MSGETSTR );
    }
  }

  /* check that we read the correct number of templates */
  if ( tmpltIndex != numTmpltString )
  {
    ABORT( status, INITSEARCHH_ETSTR, INITSEARCHH_MSGETSTR );
  }      


  /* normal exit */
  DETATCHSTATUSPTR( status );
  RETURN( status );
}
