/********************************** <lalVerbatim file="PowerConditionDataCV">
Author: Anderson, W G, Brady, P and Brown, D
$Id: ConditionData.c,v 1.19 2003/07/08 21:40:39 patrick Exp $
**************************************************** </lalVerbatim> */

#include <lal/LALStdlib.h>
#include <lal/ExcessPower.h>
#include <lal/TFTransform.h>
#include <lal/EPData.h>
#include "ConditionData.h"
#include "ExtractSeries.h"

NRCSID( CONDITIONDATAC , "power $Id: ConditionData.c,v 1.19 2003/07/08 21:40:39 patrick Exp $" );

void
LALConditionData(
    LALStatus             *status,
    LALSearchInput        *inout,
    void                  *searchParams,
    LALMPIParams          *mpiParams
    )
{
  INT4                          i;
  INT4                          dmroIndex  = 0;
  INT4                          specIndex  = 0;
  INT4                          respIndex  = 0;
  REAL4                        *dummyR4    = NULL;
  COMPLEX8                     *dummyC8    = NULL;
  EPSearchParams               *params;
    REAL4TimeSeries               series;
    REAL4Vector                   vr;
    

  INITSTATUS (status, "LALConditionData", CONDITIONDATAC);
  ATTATCHSTATUSPTR (status);

  params = (EPSearchParams *) searchParams;
  series.data = &vr;
  vr.data = NULL;

  /*
   *
   * do not condition the data on the search master
   *
   */

  if ( params->searchMaster )
  {
    /* return immediately */
    DETATCHSTATUSPTR (status);
    RETURN (status);
  }
  else
  {

    /****************************************************************
     *
     * identify multiDimData sequences by their names 
     *
     ***************************************************************/

    ASSERT (searchParams, status, CONDITIONDATAH_ENULL, CONDITIONDATAH_MSGENULL);

    for ( i = 0; i < inout->numberSequences; ++i )
    {
        if ( strstr( inout->sequences[i].name, INPUTNAME_CHANNEL))  
        {
            dmroIndex = i;
            series.data->data = inout->sequences[i].data.real4;
            series.epoch.gpsSeconds = inout->sequences[i].range.dTime.startSec;
            series.epoch.gpsNanoSeconds = inout->sequences[i].range.dTime.startNan;
            series.deltaT = inout->sequences[i].range.dTime.timeStepSize;
            series.data->length = inout->sequences[dmroIndex].dimensions[0];
        }
        else if ( strstr( inout->sequences[i].name, INPUTNAME_SPECTRUM ) )
        {
            specIndex = i;
            dummyR4   = inout->sequences[i].data.real4;
        }
        else if ( strstr( inout->sequences[i].name, INPUTNAME_RESPONSE ) )
        {
            respIndex = i;
            dummyC8   = inout->sequences[i].data.complex8;
        }
    }

    /* Check that the data was found  */
    if( series.data->data == NULL || dummyR4 == NULL || dummyC8 == NULL ){
        ABORT (status, CONDITIONDATAH_EINPUT, CONDITIONDATAH_MSGEINPUT);
    }

    /* Condition the data */
    EPConditionData( status->statusPtr, &series, searchParams);
    CHECKSTATUSPTR (status);
  }

  /****************************************************************
   * 
   * clean up and return
   *
   ****************************************************************/
  DETATCHSTATUSPTR (status);
  RETURN (status);
}


