/**** <lalVerbatim file="StochasticApplySearchHV">
Author: Anderson, W.G., Romano, J.D., and Whelan, J.T.
$Id: SGWBApplySearch.h,v 1.2 2003/09/03 02:37:39 kaice Exp $
**************************************************** </lalVerbatim> */

/******************************* <lalLaTeX file="StochasticApplySearchH">
\section{Header \texttt{SGWBApplySearch.h}}
\label{s:StochasticApplySearch.h}

Header file for a routine that implements the stochastic background
search using the standard optimally-filtered cross-correlation statistic.

\subsection*{Synopsis}
\begin{verbatim}
#include "SGWBApplySearch.h"
\end{verbatim}

\subsection*{Error conditions}
\input{StochasticApplySearchHErrTab}

\subsection*{Structures}

*************************************************** </lalLaTeX> */

#ifndef _SGWBAPPLYSEARCH_H
#define _SGWBAPPLYSEARCH_H

#include "LALWrapperInterface.h"
#include "SGWB.h"
#include "BuildDB.h"

#ifdef  __cplusplus
extern "C" {
#endif

NRCSID (SGWBAPPLYSEARCHH, "$Id: SGWBApplySearch.h,v 1.2 2003/09/03 02:37:39 kaice Exp $");

/******** <lalErrTable file="StochasticApplySearchHErrTab"> ********/
#define STOCHASTICAPPLYSEARCHH_ENULL      1
#define STOCHASTICAPPLYSEARCHH_ENNUL      2
#define STOCHASTICAPPLYSEARCHH_EMSTR      3 
#define STOCHASTICAPPLYSEARCHH_ERANK      4
#define STOCHASTICAPPLYSEARCHH_EALOC      5
#define STOCHASTICAPPLYSEARCHH_ESIZE 104
#define STOCHASTICAPPLYSEARCHH_EROWS 120

#define STOCHASTICAPPLYSEARCHH_MSGENULL   "Null pointer"
#define STOCHASTICAPPLYSEARCHH_MSGENNUL   "Non-null pointer"
#define STOCHASTICAPPLYSEARCHH_MSGEMSTR   "Search master not being used"
#define STOCHASTICAPPLYSEARCHH_MSGERANK   "Search master has incorrect rank"
#define STOCHASTICAPPLYSEARCHH_MSGEALOC   "Memory allocation error"
#define STOCHASTICAPPLYSEARCHH_MSGESIZE "Invalid data length"
#define STOCHASTICAPPLYSEARCHH_MSGEROWS "Invalid number of rows"
/******** </lalErrTable> ********/

#define STOCHASTICAPPLYSEARCHH_PROGNAME   "stochasticDSO"
#define STOCHASTICAPPLYSEARCHH_DSONAME   "libldasstochastic.so"
  /* #define STOCHASTICAPPLYSEARCHH_MIMETYPE   "Subtype 4 - cross-spectrum" */
  /* #define STOCHASTICAPPLYSEARCHH_CHANNEL     "Hanford 4km differential mode - Livingston 4km differential mode cross correlation spectrum" */
#define STOCHASTICAPPLYSEARCHH_CHANNEL     "optimally filtered cross-correlation spectrum"
#define STOCHASTICAPPLYSEARCHH_SPECTYPE    "IFO IFO differential mode cross correlation spectrum for stochastic background search"
#define STOCHASTICAPPLYSEARCHH_CCSTATREALNAME   "Cross-corr statistic for segment %d"
#define STOCHASTICAPPLYSEARCHH_CCSTATREALPARTNAME   "Real part of cross-corr statistic for segment %d"
#define STOCHASTICAPPLYSEARCHH_CCSTATIMAGPARTNAME   "Imaginary part of cross-corr statistic for segment %d"
#define STOCHASTICAPPLYSEARCHH_SIGMANAME   "Theoretical sigma of cross-corr statistic for segment %d"
#define STOCHASTICAPPLYSEARCHH_SEGLENNAME  "Length of a data segment"
#define STOCHASTICAPPLYSEARCHH_MAXSTRINGLENGTH 256

#if     WORDS_BIGENDIAN
#define STOCHASTICAPPLYSEARCHH_MIMETYPE    "x-ligo/complex-spectrum-big"
#else
#define STOCHASTICAPPLYSEARCHH_MIMETYPE    "x-ligo/complex-spectrum-little"
#endif

/******************************* <lalLaTeX file="StochasticApplySearchH">

\subsubsection*{\texttt{struct StochasticSearchOutput}}
\index{\texttt{StochasticSearchOutput}}

\noindent
The fields are:
\begin{description}
  \item[\texttt{REAL4WithUnits             *ccStatReal}]
  \item[\texttt{COMPLEX8WithUnits          *ccStatComplex}]
  \item[\texttt{REAL4WithUnits             *sigma}]
  \item[\texttt{COMPLEX8FrequencySeries    *ccSpec}]
\end{description}

*************************************************** </lalLaTeX> */

typedef struct
tagStochasticSearchOutput
{
  REAL4WithUnits             *ccStatReal;
  COMPLEX8WithUnits          *ccStatComplex;
  REAL4WithUnits             *sigma;
  COMPLEX8FrequencySeries    *ccSpec;
  UINT4                       outputType;
}
StochasticSearchOutput;

void
StochasticBuildOutput (
    LALStatus*               status,
    LALSearchOutput*         output,
    StochasticSearchOutput*  input,
    StochasticSearchParams*  params
);

void
LALCreateDBEntryBLOBU(
    LALStatus *status,
    dataBase  *output,
    LALBLOB   *blob,
    UINT4      rows
);

#ifdef  __cplusplus
}
#endif /* C++ protection */

#endif /* _STOCHASTICAPPLYSEARCH_H */


/******************************* <lalLaTeX file="StochasticApplySearchH">

\vfill{\footnotesize\input{StochasticApplySearchHV}}

\newpage\input{StochasticApplySearchC}
\newpage\input{StochasticBuildOutputC}

*************************************************** </lalLaTeX> */
