#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <lal/LALStdlib.h>
#include <lal/LALDatatypes.h>
#include <lal/Comm.h>
#include <lal/AVFactories.h>
#include <lal/TimeFreqFFT.h>
#include <lal/CoarseGrainFrequencySeries.h>
#include "SGWBApplySearch.h"
#include "SGWB.h"
#include "CastToFunctions.h"
#include "SGWBAuxFunctions.h"

NRCSID (SGWBAUXFUNCTIONSC, "$Id: SGWBAuxFunctions.c,v 1.5 2003/10/07 17:17:22 jolien Exp $");

void LALFindCheckMDSequences( LALStatus *status,
			      LALSearchInput *input,
			      StochasticSearchParams *params,
			      SequenceIndices *sI)
{

  /*
   *  
   * identify multiDimData sequences by ILWD comments, and check that 
   * the data has the correct type, dimensions, and length 
   *
   */

  INT4 i;

  sI->data1Index = -1;
  sI->data2Index = -1;
  sI->resp1Index = -1;
  sI->resp2Index = -1;
  sI->spec1Index = -1;
  sI->spec2Index = -1;

  INITSTATUS( status, "LALFINDCHECKMDSEQUENCESC", SGWBAUXFUNCTIONSC ); 
  ATTATCHSTATUSPTR (status);

  for (i = 0; i < input->numberSequences; ++i)
    { /* begin for over input->numberSequences */
       
      if ( !strcmp(input->sequences[i].comment,
		   STOCHASTICCONDITIONDATAH_RESP1COMMENT) )
	{
	  sI->resp1Index = i;
	   
	  /* check dimensions */
	  if (input->sequences[i].numberDimensions != 1)
	    {
	      ABORT (status, 
		     STOCHASTICCONDITIONDATAH_EDIMS, 
		     STOCHASTICCONDITIONDATAH_MSGEDIMS);
	    }
	   
	}
      else if ( !strcmp(input->sequences[i].comment,
			STOCHASTICCONDITIONDATAH_RESP2COMMENT) )
	{
	  sI->resp2Index = i;
	   
	  /* check dimensions */
	  if (input->sequences[i].numberDimensions != 1)
	    {
	      ABORT (status, 
		     STOCHASTICCONDITIONDATAH_EDIMS, 
		     STOCHASTICCONDITIONDATAH_MSGEDIMS);
	    }
	   
	}
      else if ( !strcmp(input->sequences[i].comment,
			STOCHASTICCONDITIONDATAH_DATA1COMMENT) )
	{
	  sI->data1Index = i;

	  /* check dimensions */
	  if (input->sequences[i].numberDimensions != 1)
	    {
	      ABORT (status, 
		     STOCHASTICCONDITIONDATAH_EDIMS, 
		     STOCHASTICCONDITIONDATAH_MSGEDIMS);
	    }
	   
	  /* check length */
	  if (input->sequences[i].dimensions[0] 
	      < params->initParams->numSegments *
	      params->initParams->numPoints)
	    {
	      ABORT (status, 
		     STOCHASTICCONDITIONDATAH_ESIZE, 
		     STOCHASTICCONDITIONDATAH_MSGESIZE);
	    }
	   
	  /* Copy time interval for search output */
	  /* dTime is of type gpsTimeInterval */
	  params->timeInterval = input->sequences[i].range.dTime;
	}
      else if ( !strcmp(input->sequences[i].comment,
			STOCHASTICCONDITIONDATAH_DATA2COMMENT) )
	{
	  sI->data2Index = i;
	   
	  /* check dimensions */
	  if (input->sequences[i].numberDimensions != 1)
	    {
	      ABORT (status, 
		     STOCHASTICCONDITIONDATAH_EDIMS, 
		     STOCHASTICCONDITIONDATAH_MSGEDIMS);
	    }
	   
	  /* check length */
	  if (input->sequences[i].dimensions[0] 
	      < params->initParams->numSegments *
	      params->initParams->numPoints)
	    {
	      ABORT (status, 
		     STOCHASTICCONDITIONDATAH_ESIZE, 
		     STOCHASTICCONDITIONDATAH_MSGESIZE);
	    }	
	}
      else if ( !strcmp(input->sequences[i].comment,
			STOCHASTICCONDITIONDATAH_SPEC1COMMENT) )
	{
	  sI->spec1Index = i;

	  /* check dimensions */
	  if (input->sequences[i].numberDimensions != 1)
	    {
	      ABORT (status, 
		     STOCHASTICCONDITIONDATAH_EDIMS, 
		     STOCHASTICCONDITIONDATAH_MSGEDIMS);
	    }
            
	}
      else if ( !strcmp(input->sequences[i].comment,
			STOCHASTICCONDITIONDATAH_SPEC2COMMENT) )
	{
	  sI->spec2Index = i;
	
	  /* check dimensions */
	  if (input->sequences[i].numberDimensions != 1)
	    {
	      ABORT (status, 
		     STOCHASTICCONDITIONDATAH_EDIMS, 
		     STOCHASTICCONDITIONDATAH_MSGEDIMS);
	    }
	
	}
      /* Have to allow for new-style calibration data */
      else if ( !strstr(input->sequences[i].name,"CAL-") )
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_ENAME, 
		 STOCHASTICCONDITIONDATAH_MSGENAME); 
	}
    } /* end for over input->numberSequences */
   
  DETATCHSTATUSPTR (status);  
  RETURN( status ); 
}





void LALAllocateParamFieldMemory( LALStatus *status,
				  StochasticSearchParams *params,
				  COMPLEX8FrequencySeries **resp1,
				  COMPLEX8FrequencySeries **resp2,
				  REAL4FrequencySeries **spec1,
				  REAL4FrequencySeries **spec2,
				  REAL4FrequencySeries **spec1Ptr,
				  REAL4FrequencySeries **spec2Ptr)
{

  INITSTATUS( status, "LALALLOCATEPARAMFIELDMEMORYC", SGWBAUXFUNCTIONSC ); 
  ATTATCHSTATUSPTR (status);

  /* 
   *
   * allocate memory for non-trivial parameter fields 
   *
   */
   
  /* spec1 */
  *spec1 = (REAL4FrequencySeries *) LALMalloc (sizeof (REAL4FrequencySeries));
  if (!(*spec1))
    {
      ABORT (status, 
	     STOCHASTICCONDITIONDATAH_EALOC, 
	     STOCHASTICCONDITIONDATAH_MSGEALOC);
    }
   
  (*spec1)->data = NULL;
  TRY( LALCreateVector (status->statusPtr,
			&((*spec1)->data),
			params->initParams->numFrequencies),
       status );
   
  /* spec2 */    
  *spec2 = (REAL4FrequencySeries *) LALMalloc (sizeof (REAL4FrequencySeries));
  if (!(*spec2))
    {
      ABORT (status, 
	     STOCHASTICCONDITIONDATAH_EALOC, 
	     STOCHASTICCONDITIONDATAH_MSGEALOC);
    }
   
  (*spec2)->data = NULL;
  TRY( LALCreateVector (status->statusPtr,
			&((*spec2)->data),
			params->initParams->numFrequencies),
       status );
    
  /* spec1Ptr */
  *spec1Ptr = (REAL4FrequencySeries *) LALMalloc (sizeof (REAL4FrequencySeries));
  if (!(*spec1Ptr))
    {
      ABORT (status, 
	     STOCHASTICCONDITIONDATAH_EALOC, 
	     STOCHASTICCONDITIONDATAH_MSGEALOC);
    }
   
  (*spec1Ptr)->data = NULL;
  TRY( LALCreateVector (status->statusPtr,
			&((*spec1Ptr)->data),
			((params->fftLength)/2)+1),
       status );
   
  /* spec2Ptr */    
  *spec2Ptr = (REAL4FrequencySeries *) LALMalloc (sizeof (REAL4FrequencySeries));
  if (!(*spec2Ptr))
    {
      ABORT (status, 
	     STOCHASTICCONDITIONDATAH_EALOC, 
	     STOCHASTICCONDITIONDATAH_MSGEALOC);
    }

  (*spec2Ptr)->data = NULL;
  TRY( LALCreateVector (status->statusPtr,
			&((*spec2Ptr)->data),
			((params->fftLength)/2)+1),
       status );
  
  /* resp1 */    
  *resp1 = (COMPLEX8FrequencySeries *) LALMalloc (sizeof (COMPLEX8FrequencySeries));
  if (!(resp1))
    {
      ABORT (status, 
	     STOCHASTICCONDITIONDATAH_EALOC, 
	     STOCHASTICCONDITIONDATAH_MSGEALOC);
    }
   
  (*resp1)->data = NULL;
  TRY( LALCCreateVector (status->statusPtr,
			 &((*resp1)->data),
			 params->initParams->numFrequencies),
       status );
   
  /* resp2 */ 
  *resp2 = (COMPLEX8FrequencySeries *) LALMalloc (sizeof (COMPLEX8FrequencySeries));
  if (!(*resp2))
    {
      ABORT (status, 
	     STOCHASTICCONDITIONDATAH_EALOC, 
	     STOCHASTICCONDITIONDATAH_MSGEALOC);
    }
   
  (*resp2)->data = NULL;
  TRY( LALCCreateVector (status->statusPtr,
			 &((*resp2)->data),
			 params->initParams->numFrequencies),
       status );
   
 
/*   *(params->fftLength) = (UINT4 *) LALMalloc( sizeof(UINT4) ); */
/*   memset( params->fftLength, 0, sizeof(UINT4) ); */

  /* 
   *
   * End allocate memory for non-trivial parameter fields 
   *
   */
   
  DETATCHSTATUSPTR (status);  
  RETURN( status ); 
  
}



void LALCalculateSpec1Spec2( LALStatus *status,
			     StochasticSearchParams *params,
			     REAL4FrequencySeries **spec1,
			     REAL4FrequencySeries **spec2,
			     REAL4FrequencySeries **spec1Ptr,
			     REAL4FrequencySeries **spec2Ptr,
			     FrequencySamplingParams freqParams)
{

  LALWindowParams                 wpars;
  AverageSpectrumParams           avgSpecParams;
  REAL4FrequencySeries            spec1Raw, spec2Raw;
  StochasticRealDataSegment*          sDummySegment = NULL;
  StochasticComplexDataSegment*       cDummySegment = NULL;
  REAL8                           startFreq;

  INITSTATUS( status, "LALCALCULATESPEC1SPEC2C", SGWBAUXFUNCTIONSC ); 
  ATTATCHSTATUSPTR (status);

  if (params->stochasticSegVec.realVec)
    { /* real option */
      /* Set up the PSD estimate calculation */
      wpars.type = Hann;
      wpars.length = params->fftLength;
	  
       printf("*wpars **** %d **** \n", wpars.length);
       printf("*** %f **** \n", freqParams.deltaF);
       printf("*** %f **** \n", params->stochasticSegVec.realVec->data->data1->deltaT);
	  
      avgSpecParams.method = useMean;
      avgSpecParams.overlap = wpars.length/2.0;
      avgSpecParams.window = NULL;
      TRY( LALCreateREAL4Window( status->statusPtr,
				 &avgSpecParams.window, 
				 &wpars ),
	   status);
      avgSpecParams.plan = NULL;
      TRY( LALCreateForwardRealFFTPlan( status->statusPtr, 
					&avgSpecParams.plan, 
					wpars.length, 
					0 ),
	   status);
      /* END: Set up the PSD estimate calculation */

      /* point dummySegment to the current segment to analyze */
      sDummySegment 
	= params->stochasticSegVec.realVec->data + params->currentSegment;
	   
      /* Welch Estimate of spec1 and spec2 using 50% overlapping
	 Hann Windows */
      TRY( LALREAL4AverageSpectrum( status->statusPtr, 
				    *spec1Ptr, 
				    sDummySegment->data1, 
				    &avgSpecParams ),
	   status);
      TRY( LALREAL4AverageSpectrum( status->statusPtr, 
				    *spec2Ptr, 
				    sDummySegment->data2, 
				    &avgSpecParams ),
	   status);
    } /* end real option */
  else
    { /* complex option */
      /* point dummySegment to the current segment to analyze */
      cDummySegment 
	= params->stochasticSegVec.complexVec->data + params->currentSegment;
	   
	   
      /* KTR: I put this abort here so that I am reminded this
	 else section is WRONG!!! */
      ABORT (status, 
	     STOCHASTICCONDITIONDATAH_EALOC, 
	     STOCHASTICCONDITIONDATAH_MSGEALOC);
	   
      /*    DO NOT DELETE */
      /********************************/
      /* Commented out on move to this function. */
      /* still need to figure out what to do here */
      /*       czpfftParams.fftPlan = params->fftPlan.complexPlan; */
      /*       czpfftParams.length  = params->initParams->fullLength; */
      /*       czpfftParams.window  = params->window1; */
      /*       TRY( LALCZeroPadAndFFT (status->statusPtr, */
      /* 			      &fseries1, */
      /* 			      cDummySegment->data1, */
      /* 			      &czpfftParams), */
      /* 	   status ); */
	   
      /*       czpfftParams.window  = params->window2; */
      /*       TRY( LALCZeroPadAndFFT (status->statusPtr, */
      /* 			      &fseries2, */
      /* 			      cDummySegment->data2, */
      /* 			      &czpfftParams), */
      /* 	   status ); */
    }/* end complex option */
       
       
       
  /* Begin move PSD Conditioning from ConditionData */

  /* 
   *
   * translate spectrum InPut to REAL4FrequencySeries
   * and coarse grain to desired resolution
   *
   */
    
  /* spec1 */

  /* KTR commented out because it will become obselete. */

  /*     if (sI.spec1Index < 0) { */
  /*         ABORT(status,  */
  /*               STOCHASTICCONDITIONDATAH_ENSEQ,  */
  /*               STOCHASTICCONDITIONDATAH_MSGENSEQ); */
  /*     } */
  /*       spec1TwoSidedData = NULL; */
  /*       TRY( LALSCreateVector(status->statusPtr, */
  /* 			    &spec1TwoSidedData, */
  /* 			    input->sequences[sI.spec1Index].dimensions[0]), */
  /* 	   status ); */

  /*       CastToReal4 ( status, spec1TwoSidedData->data, */
  /* 		    &(input->sequences[sI.spec1Index]), */
  /* 		    0, spec1TwoSidedData->length ); */

  /*       This is the physical starting frequency */ 
  /*       startFreq = input->sequences[sI.spec1Index].range.dFreq.baseFreq */
  /* 	+ input->sequences[sI.spec1Index].range.dFreq.startFreq; */

  startFreq = (*spec1Ptr)->f0;

  /* check if the spectrum includes negative frequencies */
  if ( startFreq >= 0) {
    /* 	spec1Raw.data = spec1TwoSidedData; */
    spec1Raw.data = (*spec1Ptr)->data;
    spec1Raw.f0 = startFreq;
  }
  else 
    {
      /* code here is wrong so abort */
      ABORT (status, 
	     STOCHASTICCONDITIONDATAH_EALOC, 
	     STOCHASTICCONDITIONDATAH_MSGEALOC);

      /* END code here is wrong so abort */

      /*    DO NOT DELETE */
      /********************************/
      /* Commented out on move to this function. */
      /* still need to figure out what to do here */
      /*       offset  */
      /* 	= (INT4) (- startFreq */
      /* 		  / input->sequences[sI.spec1Index].range.dFreq.freqStepSize */
      /* 		  + 0.5); */
      /*       spec1Raw.data = (REAL4Vector *) */
      /* 	LALMalloc(sizeof(REAL4Vector)); */

      /*       if (!(spec1Raw.data)) */
      /* 	{ */
      /* 	  ABORT (status,  */
      /* 		 STOCHASTICCONDITIONDATAH_EALOC,  */
      /* 		 STOCHASTICCONDITIONDATAH_MSGEALOC); */
      /* 	} */

      /* 	  spec1Raw.data->length = spec1TwoSidedData->length - offset; */
      /* 	  spec1Raw.data->data = spec1TwoSidedData->data + offset; */
      /* 	  spec1Raw.f0 = 0.0; */
    }

  /*       strncpy (spec1Raw.name,  */
  /* 	       input->sequences[sI.spec1Index].name, */
  /* 	       LALNameLength * sizeof(CHAR)); */

  /*       spec1Raw.deltaF =  */
  /* 	input->sequences[sI.spec1Index].range.dFreq.freqStepSize; */

  /*       spec1Raw.epoch.gpsSeconds =  */
  /* 	(input->sequences[sI.spec1Index].range.dFreq.gpsStartTimeSec); */

  /*       spec1Raw.epoch.gpsNanoSeconds =  */
  /* 	(input->sequences[sI.spec1Index].range.dFreq.gpsStartTimeNan); */

  strncpy (spec1Raw.name, 
	   (*spec1Ptr)->name,
	   LALNameLength * sizeof(CHAR));
      
  spec1Raw.deltaF = (*spec1Ptr)->deltaF;
      
  spec1Raw.epoch.gpsSeconds = (*spec1Ptr)->epoch.gpsSeconds;
      
  spec1Raw.epoch.gpsNanoSeconds = (*spec1Ptr)->epoch.gpsNanoSeconds;

  /*       printf("\n\n ****** \n"); */
  /*       for (j = 0; j <  spec1Raw.data->length; j++) { */
  /* 	printf(" %e %i %i  \n", *(spec1Raw.data->data+j), j, params->currentSegment); */
  /*       } */
  /*       printf("\n ******  \n\n"); */


  /* really should get this from the ILWD */
  /*
    if ( strncmp(input->sequences[sI.spec1Index].units,"counts^2/Hz",
    maxMultiDimUnits) && 
    strncmp(input->sequences[sI.spec1Index].units,"count^2/Hz",
    maxMultiDimUnits) &&
    strncmp(input->sequences[sI.spec1Index].units,"counts^2/hz",
    maxMultiDimUnits) && 
    strncmp(input->sequences[sI.spec1Index].units,"count^2/hz",
    maxMultiDimUnits) && 
    strncmp(input->sequences[sI.spec1Index].units,"s count^2",
    maxMultiDimUnits) &&
    strncmp(input->sequences[sI.spec1Index].units,
    input->sequences[sI.data1Index].units,
    maxMultiDimUnits) ) {
    ABORT (status, 
    STOCHASTICCONDITIONDATAH_EUNIT, 
    STOCHASTICCONDITIONDATAH_MSGEUNIT);
    }
  */

  spec1Raw.sampleUnits = lalDimensionlessUnit;
  spec1Raw.sampleUnits.unitNumerator[LALUnitIndexADCCount] = 2;
  spec1Raw.sampleUnits.unitNumerator[LALUnitIndexSecond]   = 1;

  TRY( LALSCoarseGrainFrequencySeries(status->statusPtr,
				      *spec1,
				      &spec1Raw,
				      &freqParams),
       status );

  /*       if (spec1Raw.data != spec1TwoSidedData) { */
  /* 	LALFree(spec1Raw.data); */
  /*       } */

  /*       TRY( LALSDestroyVector(status->statusPtr, */
  /* 			     &spec1TwoSidedData), */
  /* 	   status );     */

  /* spec2 */

  /* KTR commented out because it will become obselete. */
    
  /*     if (sI.spec2Index < 0) { */
  /*         ABORT(status,  */
  /*               STOCHASTICCONDITIONDATAH_ENSEQ,  */
  /*               STOCHASTICCONDITIONDATAH_MSGENSEQ); */
  /*     } */
  /*       spec2TwoSidedData = NULL; */
  /*       TRY( LALSCreateVector(status->statusPtr, */
  /* 			    &spec2TwoSidedData, */
  /* 			    input->sequences[sI.spec2Index].dimensions[0]), */
  /* 	   status ); */

  /*       CastToReal4 ( status, spec2TwoSidedData->data, */
  /* 		    &(input->sequences[sI.spec2Index]), */
  /* 		    0, spec2TwoSidedData->length ); */

  /* This is the physical starting frequency */
  /*      startFreq = input->sequences[sI.spec2Index].range.dFreq.baseFreq */
  /* 	+ input->sequences[sI.spec2Index].range.dFreq.startFreq; */


  startFreq = (*spec2Ptr)->f0;

  /* check if the spectrum includes negative frequencies */
  if ( startFreq >= 0) {
    /* 	spec2Raw.data = spec2TwoSidedData; */
    spec2Raw.data = (*spec2Ptr)->data;
    spec2Raw.f0 = startFreq;
  }
  else 
    {
      /* code here is wrong so abort */
      ABORT (status, 
	     STOCHASTICCONDITIONDATAH_EALOC, 
	     STOCHASTICCONDITIONDATAH_MSGEALOC);

      /* END code here is wrong so abort */

      /* END code here is wrong so abort */

      /*    DO NOT DELETE */
      /********************************/
      /* Commented out on move to this function. */
      /* still need to figure out what to do here */
      /*      offset  */
      /* 	= (INT4) (- startFreq */
      /* 		  / input->sequences[sI.spec2Index].range.dFreq.freqStepSize */
      /* 		  + 0.5); */
      /*       spec2Raw.data = (REAL4Vector *) */
      /* 	LALMalloc(sizeof(REAL4Vector)); */
	  
      /*       if (!(spec2Raw.data)) */
      /* 	{ */
      /* 	  ABORT (status,  */
      /* 		 STOCHASTICCONDITIONDATAH_EALOC,  */
      /* 		 STOCHASTICCONDITIONDATAH_MSGEALOC); */
      /* 	} */

      /* 	  spec2Raw.data->length = spec2TwoSidedData->length - offset; */
      /* 	  spec2Raw.data->data = spec2TwoSidedData->data + offset; */
      /* 	  spec2Raw.f0 = 0.0; */
    }

  /*       strncpy (spec2Raw.name,  */
  /* 	       input->sequences[sI.spec2Index].name, */
  /* 	       LALNameLength * sizeof(CHAR)); */

  /*       spec2Raw.deltaF =  */
  /* 	input->sequences[sI.spec2Index].range.dFreq.freqStepSize; */

  /*       spec2Raw.epoch.gpsSeconds =  */
  /* 	(input->sequences[sI.spec2Index].range.dFreq.gpsStartTimeSec); */

  /*       spec2Raw.epoch.gpsNanoSeconds =  */
  /* 	(input->sequences[sI.spec2Index].range.dFreq.gpsStartTimeNan); */

  strncpy (spec2Raw.name, 
	   (*spec2Ptr)->name,
	   LALNameLength * sizeof(CHAR));
      
  spec2Raw.deltaF = (*spec2Ptr)->deltaF;
      
  spec2Raw.epoch.gpsSeconds = (*spec2Ptr)->epoch.gpsSeconds;
      
  spec2Raw.epoch.gpsNanoSeconds = (*spec2Ptr)->epoch.gpsNanoSeconds;

  /* really should get this from the ILWD */
  /*
    if ( strncmp(input->sequences[sI.spec2Index].units,"counts^2/Hz",
    maxMultiDimUnits) && 
    strncmp(input->sequences[sI.spec2Index].units,"count^2/Hz",
    maxMultiDimUnits) &&
    strncmp(input->sequences[sI.spec2Index].units,"counts^2/hz",
    maxMultiDimUnits) && 
    strncmp(input->sequences[sI.spec2Index].units,"count^2/hz",
    maxMultiDimUnits) && 
    strncmp(input->sequences[sI.spec2Index].units,"s count^2",
    maxMultiDimUnits) &&
    strncmp(input->sequences[sI.spec2Index].units,
    input->sequences[sI.data2Index].units,
    maxMultiDimUnits) ) {
    ABORT (status, 
    STOCHASTICCONDITIONDATAH_EUNIT, 
    STOCHASTICCONDITIONDATAH_MSGEUNIT);
    }
  */

  spec2Raw.sampleUnits = lalDimensionlessUnit;
  spec2Raw.sampleUnits.unitNumerator[LALUnitIndexADCCount] = 2;
  spec2Raw.sampleUnits.unitNumerator[LALUnitIndexSecond]   = 1;

  /*       printf("\n\n ****** \n"); */
  /*       for (j = 0; j <  spec2Raw.data->length; j++) { */
  /*  printf(" %e %i %i  \n", *(spec2Raw.data->data+j), j, params->currentSegment); */
  /*       } */
  /*       printf("\n ******  \n\n"); */

  TRY( LALSCoarseGrainFrequencySeries(status->statusPtr,
				      *spec2,
				      &spec2Raw,
				      &freqParams),
       status );


  /*       printf("\n\n ****** \n"); */
  /*       for (j = 0; j <  spec2->data->length; j++) { */
  /* 	printf(" %e %e %i %i  \n", spec1->data->data[j], spec2->data->data[j], j, params->currentSegment); */
  /*       } */
  /*       printf("\n ******  \n\n"); */

  /*       if (spec2Raw.data != spec2TwoSidedData) { */
  /* 	LALFree(spec2Raw.data); */
  /*       } */

  /*       TRY( LALSDestroyVector(status->statusPtr, */
  /* 			     &spec2TwoSidedData), */
  /* 	   status );     */
  /* End move PSD conditioning */

  TRY( LALDestroyREAL4Window( status->statusPtr, &avgSpecParams.window),
       status );
 
  TRY( LALDestroyRealFFTPlan( status->statusPtr, &avgSpecParams.plan ),
       status );

  DETATCHSTATUSPTR (status);  
  RETURN( status ); 
}


void LALCalculateResp1Resp2( LALStatus *status,
			     LALSearchInput *input,
			     StochasticSearchParams *params,
			     COMPLEX8FrequencySeries **resp1, 
			     COMPLEX8FrequencySeries **resp2,
			     FrequencySamplingParams freqParams,
			     SequenceIndices sI)
{
  COMPLEX8FrequencySeries         resp1Raw, resp2Raw;
  INT4                            increment;
  COMPLEX8Vector                 *resp1TwoSidedData, *resp2TwoSidedData;
  CHAR                            ifo[3];
  INT4                            offset;
  REAL8                           startFreq;
  INT8                            epochNanoSeconds;


  INITSTATUS( status, "LALCALCULATERESP1RESP2C", SGWBAUXFUNCTIONSC ); 
  ATTATCHSTATUSPTR (status);


  fprintf(stdout,"AUX slavecount %d nodenumsegs %d currentseg %d \n", params->slaveCounter, params->initParams->nodeNumSegs, params->currentSegment); 
  fflush( stdout ); 


  /*
   *  
   *  translate response InPut to COMPLEX8FrequencySeries
   *  and coarse grain to desired resolution
   *
   */

  /* resp1 */
  if ( sI.resp1Index < 0 )
    {
      /* *** OLD *** Set epoch to the middle of the job */
      /* Set epoch to the middle of the current segment */

      epochNanoSeconds = 1000000000L
	* (INT8) input->sequences[sI.data1Index].range.dTime.startSec;
      epochNanoSeconds 
	+= (INT8) input->sequences[sI.data1Index].range.dTime.startNan;
      increment = params->currentSegment * params->segmentDuration;
      epochNanoSeconds += 1000000000L * (INT8) increment;
      epochNanoSeconds += 1000000000L * (INT8) (params->segmentDuration/2);
      
      resp1Raw.epoch.gpsSeconds
	= (INT4) (epochNanoSeconds / 1000000000L);
      printf("\n *** time *** %d %d %d\n", resp1Raw.epoch.gpsSeconds, params->currentSegment, params->slaveCounter );
      resp1Raw.epoch.gpsNanoSeconds
	= (INT4) (epochNanoSeconds % 1000000000L);
      
      /* Set output units to counts per attostrain */

      resp1Raw.sampleUnits = lalADCCountUnit;
      resp1Raw.sampleUnits.unitNumerator[LALUnitIndexStrain]   = -1;
      resp1Raw.sampleUnits.powerOfTen = 18;

      /* LALExtractResponse only works with f0=0, so have to request
       * more data and then coarse-grain; this choice makes sure the
       * last frequency sample in the response function is at or
       * beyond the last sample in the ultimately desired freq series
       */

      resp1Raw.f0 = 0;
      resp1Raw.deltaF = freqParams.deltaF;
      resp1Raw.data = NULL;

      TRY ( LALCCreateVector(status->statusPtr, &(resp1Raw.data),
			     params->initParams->numFrequencies
			     + ceil(freqParams.f0/freqParams.deltaF)),
	    status );

      ifo[0] = input->sequences[sI.data1Index].range.dTime.channelName[0][0];
      ifo[1] = input->sequences[sI.data1Index].range.dTime.channelName[0][1];
      ifo[2] = '\0';

      /*  #ifdef STOCHASTIC_SO_DEBUG */
      printf("#%s#\n",ifo);

      printf("This Far %d sec %d nan\n", resp1Raw.epoch.gpsSeconds,
	     resp1Raw.epoch.gpsNanoSeconds);
      /*  #endif */

      TRY ( LALExtractResponse(status->statusPtr,
			       &resp1Raw, 
			       input, 
			       ifo),
	    status );
      
      TRY( LALCCoarseGrainFrequencySeries(status->statusPtr,
					  *resp1,
					  &resp1Raw,
					  &freqParams),
	   status );

      TRY ( LALCDestroyVector(status->statusPtr, &(resp1Raw.data)),
	    status );
      
    }
  else
    {
      resp1TwoSidedData = NULL;
      TRY( LALCCreateVector(status->statusPtr,
			    &resp1TwoSidedData,
			    input->sequences[sI.resp1Index].dimensions[0]),
	   status );
      
      CastToComplex8 ( status, resp1TwoSidedData->data,
		       &(input->sequences[sI.resp1Index]),
		       0, resp1TwoSidedData->length );

      startFreq = input->sequences[sI.resp1Index].range.dFreq.baseFreq
	+ input->sequences[sI.resp1Index].range.dFreq.startFreq;
      
      /* need to discard redundant negative frequencies to make
       * LALCCoarseGrainFrequencySeries happy 
       */
      
      if (startFreq >= 0)
	{
	  resp1Raw.data = resp1TwoSidedData;
	  resp1Raw.f0 =startFreq;
	}
      else 
	{
	  offset 
	    = (INT4) (-startFreq
		      / input->sequences[sI.resp1Index].range.dFreq.freqStepSize
		      + 0.5);
	  resp1Raw.data = (COMPLEX8Vector *)
	    LALMalloc(sizeof(COMPLEX8Vector));
	  
	  if (!(resp1Raw.data))
	    {
	      ABORT (status, 
		     STOCHASTICCONDITIONDATAH_EALOC, 
		     STOCHASTICCONDITIONDATAH_MSGEALOC);
	    }
	  
	  resp1Raw.data->length = resp1TwoSidedData->length - offset;
	  resp1Raw.data->data = resp1TwoSidedData->data + offset;
	  resp1Raw.f0 = 0.0;
	}
      
      strncpy (resp1Raw.name, 
	       input->sequences[sI.resp1Index].name,
	       LALNameLength * sizeof(CHAR));
      
      resp1Raw.deltaF = 
	input->sequences[sI.resp1Index].range.dFreq.freqStepSize;
      
      resp1Raw.epoch.gpsSeconds = 
	(input->sequences[sI.resp1Index].range.dFreq.gpsStartTimeSec);
      
      resp1Raw.epoch.gpsNanoSeconds = 
	(input->sequences[sI.resp1Index].range.dFreq.gpsStartTimeNan);
      
      /* really should get this from the ILWD */
      /*
	if ( strncmp(input->sequences[sI.resp1Index].units,"count/attostrain",
	maxMultiDimUnits) && 
	strncmp(input->sequences[sI.resp1Index].units,"10^18 ADC count/strain",
	maxMultiDimUnits) && 
	strncmp(input->sequences[sI.resp1Index].units,"10^18 strain^-1 count",
	maxMultiDimUnits) ) {
	ABORT (status, 
	STOCHASTICCONDITIONDATAH_EUNIT, 
	STOCHASTICCONDITIONDATAH_MSGEUNIT);
	}
      */
      resp1Raw.sampleUnits = lalADCCountUnit;
      resp1Raw.sampleUnits.unitNumerator[LALUnitIndexStrain]   = -1;
      resp1Raw.sampleUnits.powerOfTen = 18;
      
      TRY( LALCCoarseGrainFrequencySeries(status->statusPtr,
					  *resp1,
					  &resp1Raw,
					  &freqParams),
	   status );
      
      if (resp1Raw.data != resp1TwoSidedData)
	{
	  LALFree(resp1Raw.data);
	}
      
      TRY( LALCDestroyVector(status->statusPtr,
			     &resp1TwoSidedData),
	   status );    
    }

  /* resp2 */
  if ( sI.resp2Index < 0 )
    {
      /* Set epoch to the middle of the current segment */

      epochNanoSeconds = 1000000000L
	* (INT8) input->sequences[sI.data2Index].range.dTime.startSec;
      epochNanoSeconds 
	+= (INT8) input->sequences[sI.data2Index].range.dTime.startNan;
      increment = params->currentSegment * params->segmentDuration;
      epochNanoSeconds += 1000000000L * (INT8) increment;
      epochNanoSeconds += 1000000000L * (INT8) (params->segmentDuration/2);

      resp2Raw.epoch.gpsSeconds
	= (INT4) (epochNanoSeconds / 1000000000L);
      resp2Raw.epoch.gpsNanoSeconds
	= (INT4) (epochNanoSeconds % 1000000000L);

      /* Set output units to counts per attostrain */

      resp2Raw.sampleUnits = lalADCCountUnit;
      resp2Raw.sampleUnits.unitNumerator[LALUnitIndexStrain]   = -1;
      resp2Raw.sampleUnits.powerOfTen = 18;


      /* LALExtractResponse only works with f0=0, so have to request
       * more data and then coarse-grain; this choice makes sure the
       * last frequency sample in the response function is at or
       * beyond the last sample in the ultimately desired freq series
       */

      resp2Raw.f0 = 0;
      resp2Raw.deltaF = freqParams.deltaF;
      resp2Raw.data = NULL;

      TRY ( LALCCreateVector(status->statusPtr, &(resp2Raw.data),
			     params->initParams->numFrequencies
			     + ceil(freqParams.f0/freqParams.deltaF)),
	    status );

      ifo[0] = input->sequences[sI.data2Index].range.dTime.channelName[0][0];
      ifo[1] = input->sequences[sI.data2Index].range.dTime.channelName[0][1];
      ifo[2] = '\0';

#ifdef STOCHASTIC_SO_DEBUG
      printf("#%s#\n",ifo);

      printf("%d sec %d nan\n", resp2Raw.epoch.gpsSeconds,
	     resp2Raw.epoch.gpsNanoSeconds);
#endif

      TRY ( LALExtractResponse(status->statusPtr, &resp2Raw, input, ifo),
	    status );
      
      TRY( LALCCoarseGrainFrequencySeries(status->statusPtr,
					  *resp2,
					  &resp2Raw,
					  &freqParams),
	   status );

      TRY ( LALCDestroyVector(status->statusPtr, &(resp2Raw.data)),
	    status );
    }
  else
    {
      resp2TwoSidedData = NULL;
      TRY( LALCCreateVector(status->statusPtr,
			    &resp2TwoSidedData,
			    input->sequences[sI.resp2Index].dimensions[0]),
	   status );
      
      CastToComplex8 ( status, resp2TwoSidedData->data,
		       &(input->sequences[sI.resp2Index]),
		       0, resp2TwoSidedData->length );

      startFreq = input->sequences[sI.resp2Index].range.dFreq.baseFreq
	+ input->sequences[sI.resp2Index].range.dFreq.startFreq;

      /* need to discard redundant negative frequencies to make
       * LALCCoarseGrainFrequencySeries happy 
       */

      if ( startFreq >= 0)
	{
	  resp2Raw.data = resp2TwoSidedData;
	  resp2Raw.f0 = startFreq;
	}
      else 
	{
	  offset 
	    = (INT4) (- startFreq
		      / input->sequences[sI.resp2Index].range.dFreq.freqStepSize
		      + 0.5);
	  resp2Raw.data = (COMPLEX8Vector *)
	    LALMalloc(sizeof(COMPLEX8Vector));
	
	  if (!(resp2Raw.data))
	    {
	      ABORT (status, 
		     STOCHASTICCONDITIONDATAH_EALOC, 
		     STOCHASTICCONDITIONDATAH_MSGEALOC);
	    }
	
	  resp2Raw.data->length = resp2TwoSidedData->length - offset;
	  resp2Raw.data->data = resp2TwoSidedData->data + offset;
	  resp2Raw.f0 = 0.0;
	}
      
      strncpy (resp2Raw.name, 
	       input->sequences[sI.resp2Index].name,
	       LALNameLength * sizeof(CHAR));
      
      resp2Raw.deltaF = 
	input->sequences[sI.resp2Index].range.dFreq.freqStepSize;
      
      resp2Raw.epoch.gpsSeconds = 
	(input->sequences[sI.resp2Index].range.dFreq.gpsStartTimeSec);
      
      resp2Raw.epoch.gpsNanoSeconds = 
	(input->sequences[sI.resp2Index].range.dFreq.gpsStartTimeNan);

      /* really should get this from the ILWD */
      /*
	if ( strncmp(input->sequences[sI.resp2Index].units,"count/attostrain",
	maxMultiDimUnits) && 
	strncmp(input->sequences[sI.resp2Index].units,"10^18 ADC count/strain",
	maxMultiDimUnits) && 
	strncmp(input->sequences[sI.resp2Index].units,"10^18 strain^-1 count",
	maxMultiDimUnits) ) {
	ABORT (status, 
	STOCHASTICCONDITIONDATAH_EUNIT, 
	STOCHASTICCONDITIONDATAH_MSGEUNIT);
	}
      */
      resp2Raw.sampleUnits = lalADCCountUnit;
      resp2Raw.sampleUnits.unitNumerator[LALUnitIndexStrain]   = -1;
      resp2Raw.sampleUnits.powerOfTen = 18;
      
      TRY( LALCCoarseGrainFrequencySeries(status->statusPtr,
					  *resp2,
					  &resp2Raw,
					  &freqParams),
	   status );
      
      if (resp2Raw.data != resp2TwoSidedData)
	{
	  LALFree(resp2Raw.data);
	}
      
      TRY( LALCDestroyVector(status->statusPtr,
			     &resp2TwoSidedData),
	   status );
    }

  /*        printf("\n\n ****** \n");  */
  /*        for (j = 0; j <  spec2->data->length; j++) {  */
  /*  	printf(" %e %e %i %i  \n", spec1->data->data[j], spec2->data->data[j], j, params->currentSegment);  */
  /*        }  */
  /*        printf("\n ******  \n\n");  */


  /* End move Response conditioning from ConditionData */

  DETATCHSTATUSPTR (status);  
  RETURN( status ); 
} /* end calcResp1Resp2 */



void LALAssignDataPointers( LALStatus *status,
			    LALSearchInput *inout,
			    StochasticSearchParams *params,
			    SequenceIndices sI)
{

  INT4                            i;
  INT8                            epochNanoSeconds;
  StochasticRealDataSegmentVector     *sDataSegVec      = NULL;
  StochasticRealDataSegment           *sDummySegment    = NULL;
  StochasticComplexDataSegmentVector  *cDataSegVec      = NULL;
  StochasticComplexDataSegment        *cDummySegment    = NULL;

  INITSTATUS( status, "LALASSIGNDATAPOINTERSC", SGWBAUXFUNCTIONSC ); 
  ATTATCHSTATUSPTR (status);

  /* KTR if in the future we want to calculate time shifts, then
     this section of code should be moved to a separate function.
     The initial step would be to change the above requirement
     that data streams be of length numSegments * numPoints.
     The pointer below would then be moved to a start time which
     centered the time series (i.e. equal overhang on the ends).
     This would allow shifts in to the data. Overhangs would need
     to be >= 0.5*deltaT*numShifts. 
  */
  /* begin time shift move */

  /* 
   * translate InPut data to DataSegment 
   * Copy data from multiDim structure to the
   * params structure i.e. params->stochasticSegVec real or imaginary.
   */ 
      
  if (!(params->initParams->baseFreq))
    {   /*  Real data*/
	
      /* This check is mostly to make sure non-heterodyned data are real */
      if (inout->sequences[sI.data1Index].type != real_4) 
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_ETYPE, 
		 STOCHASTICCONDITIONDATAH_MSGETYPE);
	}
      if (inout->sequences[sI.data2Index].type != real_4) 
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_ETYPE, 
		 STOCHASTICCONDITIONDATAH_MSGETYPE);
	}
	      
      sDataSegVec = params->stochasticSegVec.realVec;
	
      for (i = 0; i < sDataSegVec->length; ++i)
	{ /* begin for over real data segments */
	  /* 	  printf(" len **** %d *****\n", sDataSegVec->length); */
	  /* point to current segment */
	  sDummySegment = sDataSegVec->data + i;
	
	  /* 
	   * copy the data 
	   */
	  
	  /* data1 */
	  CastToReal4 ( status, sDummySegment->data1->data->data,
			&(inout->sequences[sI.data1Index]),
			i * params->initParams->numPoints,
			params->initParams->numPoints );
	  
	  sDummySegment->data1->data->length = params->initParams->numPoints;
	  
	  strncpy (sDummySegment->data1->name,
		   inout->sequences[sI.data1Index].name,
		   LALNameLength * sizeof(CHAR));
	  
	  sDummySegment->data1->deltaT = 
	    inout->sequences[sI.data1Index].range.dTime.timeStepSize;
	  
	  epochNanoSeconds = 1000000000L
	    * (INT8) inout->sequences[sI.data1Index].range.dTime.startSec;
	  epochNanoSeconds 
	    += (INT8) inout->sequences[sI.data1Index].range.dTime.startNan;
	  epochNanoSeconds
	    += (INT8) ( 1e9 * (REAL8) i 
			* params->segmentDuration );
	  
	  sDummySegment->data1->epoch.gpsSeconds
	    = (INT4) (epochNanoSeconds / 1000000000L);
	  sDummySegment->data1->epoch.gpsNanoSeconds
	    = (INT4) (epochNanoSeconds % 1000000000L);
	  
	  sDummySegment->data1->f0 = 0.0;
	  
	  /* really should get this from the ILWD */
	  
	  sDummySegment->data1->sampleUnits = lalADCCountUnit;
	  
	  /* data2 */
	  CastToReal4 ( status, sDummySegment->data2->data->data,
			&(inout->sequences[sI.data2Index]),
			i * params->initParams->numPoints,
			params->initParams->numPoints );
	  
	  sDummySegment->data2->data->length = params->initParams->numPoints;
	  
	  strncpy (sDummySegment->data2->name,
		   inout->sequences[sI.data2Index].name,
		   LALNameLength * sizeof(CHAR));
	  
	  sDummySegment->data2->deltaT = 
	    inout->sequences[sI.data2Index].range.dTime.timeStepSize;
	  
	  epochNanoSeconds = 1000000000L
	    * (INT8) inout->sequences[sI.data2Index].range.dTime.startSec;
	  epochNanoSeconds 
	    += (INT8) inout->sequences[sI.data2Index].range.dTime.startNan;
	  epochNanoSeconds
	    += (INT8) ( 1e9 * (REAL8) i 
			* params->segmentDuration );
	  
	  sDummySegment->data2->epoch.gpsSeconds
	    = (INT4) (epochNanoSeconds / 1000000000L);
	  sDummySegment->data2->epoch.gpsNanoSeconds
	    = (INT4) (epochNanoSeconds % 1000000000L);
	  
	  sDummySegment->data2->f0 = 0.0;
	  
	  /* really should get this from the ILWD */
	  
	  sDummySegment->data2->sampleUnits = lalADCCountUnit;
	}
    } /* end real data option */
  else 
    {   /*  Complex data*/
      /* This check is mostly to make sure heterodyned data are complex */
      if (inout->sequences[sI.data1Index].type != complex_8) 
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_ETYPE, 
		 STOCHASTICCONDITIONDATAH_MSGETYPE);
	}
      if (inout->sequences[sI.data2Index].type != complex_8) 
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_ETYPE, 
		 STOCHASTICCONDITIONDATAH_MSGETYPE);
	}

      cDataSegVec = params->stochasticSegVec.complexVec;

      for (i = 0; i < cDataSegVec->length; ++i)
	{ /* begin for over complex data segments */
	  /* point to current segment */
	  cDummySegment = cDataSegVec->data + i;
	  
	  /* 
	   * copy the data 
	   */
	  
	  /* data1 */
	  CastToComplex8 ( status, cDummySegment->data1->data->data,
			   &(inout->sequences[sI.data1Index]),
			   i * params->initParams->numPoints,
			   params->initParams->numPoints );
	  
	  cDummySegment->data1->data->length = params->initParams->numPoints;
	  
	  strncpy (cDummySegment->data1->name,
		   inout->sequences[sI.data1Index].name,
		   LALNameLength * sizeof(CHAR));
	  
	  cDummySegment->data1->deltaT = 
	    inout->sequences[sI.data1Index].range.dTime.timeStepSize;
	  
	  epochNanoSeconds = 1000000000L
	    * (INT8) inout->sequences[sI.data1Index].range.dTime.startSec;
	  epochNanoSeconds 
	    += (INT8) inout->sequences[sI.data1Index].range.dTime.startNan;
	  epochNanoSeconds
	    += (INT8) ( 1e9 * (REAL8) i 
			* params->segmentDuration );

	  cDummySegment->data1->epoch.gpsSeconds
	    = (INT4) (epochNanoSeconds / 1000000000L);
	  cDummySegment->data1->epoch.gpsNanoSeconds
	    = (INT4) (epochNanoSeconds % 1000000000L);
  
	  cDummySegment->data1->f0 = params->initParams->baseFreq;
	  
	  /* really should get this from the ILWD */
	  
	  cDummySegment->data1->sampleUnits = lalADCCountUnit;
	  
	  /* data2 */
	  CastToComplex8 ( status, cDummySegment->data2->data->data,
			   &(inout->sequences[sI.data2Index]),
			   i * params->initParams->numPoints,
			   params->initParams->numPoints );
	  
	  cDummySegment->data2->data->length = params->initParams->numPoints;
	  
	  strncpy (cDummySegment->data2->name,
		   inout->sequences[sI.data2Index].name,
		   LALNameLength * sizeof(CHAR));
	  
	  cDummySegment->data2->deltaT = 
	    inout->sequences[sI.data2Index].range.dTime.timeStepSize;
	  
	  epochNanoSeconds = 1000000000L
	    * (INT8) inout->sequences[sI.data2Index].range.dTime.startSec;
	  epochNanoSeconds 
	    += (INT8) inout->sequences[sI.data2Index].range.dTime.startNan;
	  epochNanoSeconds
	    += (INT8) ( 1e9 * (REAL8) i 
			* params->segmentDuration );
	  
	  cDummySegment->data2->epoch.gpsSeconds
	    = (INT4) (epochNanoSeconds / 1000000000L);
	  cDummySegment->data2->epoch.gpsNanoSeconds
	    = (INT4) (epochNanoSeconds % 1000000000L);
	  
	  cDummySegment->data2->f0 = params->initParams->baseFreq;
	  
	  /* really should get this from the ILWD */
	  
	  cDummySegment->data2->sampleUnits = lalADCCountUnit;
	}
    } /* end complex data option */

  DETATCHSTATUSPTR (status);  
  RETURN( status ); 
}




