/*----------------------------------------------------------------------- 
 * 
 * File Name: BuildOutput.c
 *
 * Author: Brown, D. A.
 * 
 * Revision: $Id: BuildOutput.c,v 1.2 2001/08/27 17:23:23 patrick Exp $
 * 
 *-----------------------------------------------------------------------
 */

#include "ApplySearch.h"

NRCSID( BUILDOUTPUTC, "$Id: BuildOutput.c,v 1.2 2001/08/27 17:23:23 patrick Exp $" );

void
BuildFakeInspiralOutPut (
    LALStatus          *status,
    LALSearchOutput    *output,
    UINT4               numEvents
                    )
{
  outPut               *outputArray;
  dataBase             *prev;
  dataBase             *dbEntry;
  UINT4                 i;


  INITSTATUS (status, "BuildFakeInspiralOutPut", BUILDOUTPUTC);


  /*
   *
   * create memory for the outPut array
   *
   */


  output->numOutput = numEvents;

  outputArray = output->result = (outPut *)
    LALCalloc( 1, numEvents * sizeof(outPut) );
  if ( ! outputArray )
  {
    ABORT( status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC );
  }


  /*
   *
   * fill the array of output
   *
   */


  for ( i = 0; i < numEvents; ++i )
  {

    /* this is the same for every output                           */
    /* the serach is an inspiral search, the output is significant */
    /* (meaning that we want it to go in the database)             */
    /* there is no stateVector or optional output data created     */
    outputArray[i].search         = binaryInspiral;
    outputArray[i].significant    = 1;
    outputArray[i].states         = NULL;
    outputArray[i].optional       = NULL;

    /* template index number: dummy value */
    outputArray[i].indexNumber = 0;


    /* 
     *
     * alloc memory for the database linked list
     *
     */


    /* only create the entries that are required by the dataBase. see */
    /* /ldas/ldas-0.0/doc/db2/doc/text/sngl_inspiral.sql for details  */

    /* ifo name is the head of the linked list */
    dbEntry = outputArray[i].results = 
      (dataBase *) LALMalloc( sizeof(dataBase) );
    if ( ! dbEntry )
    {
      ABORT( status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC );
    }
    dbEntry->previous = NULL;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "ifo" );

    dbEntry->type       = char_s_ptr;
    dbEntry->numberRows = 1;

    dbEntry->rows.chars = 
      (CHAR *) LALMalloc( 2 * dbEntry->numberRows * sizeof(CHAR) );
    if ( ! dbEntry->rows.chars )
    {
      ABORT( status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC );
    }
    dbEntry->rows.chars[0] = 'L';
    dbEntry->rows.chars[1] = '1';

    dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
    if ( ! dbEntry->rowDimensions )
    {
      ABORT( status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC );
    }
    *(dbEntry->rowDimensions) = 2;

    prev = dbEntry;

    /* search */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    if ( ! dbEntry )
    {
      ABORT( status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC );
    }
    dbEntry->previous = prev;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "search" );

    dbEntry->type       = char_s_ptr;
    dbEntry->numberRows = 1;

    /* scope the blobStr variable... */
    {
      UINT4 j = 0;
      const CHAR blobStr[] = "template";

      dbEntry->rows.chars = (CHAR *) 
        LALMalloc( strlen( blobStr ) * sizeof(CHAR) );
      if ( ! dbEntry->rows.chars )
      {
        ABORT( status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC );
      }
      while ( blobStr[j] ) dbEntry->rows.chars[j++] = blobStr[j];

      dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
      if ( ! dbEntry->rowDimensions )
      {
        ABORT( status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC );
      }
      *(dbEntry->rowDimensions) = (UINT4) strlen( blobStr );
    }

    prev = dbEntry;

    /* end time seconds */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    if ( ! dbEntry )
    {
      ABORT( status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC );
    }
    dbEntry->previous = prev;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "end_time" );

    dbEntry->type       = int_4s;
    dbEntry->numberRows = 1;
    dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
    if ( ! dbEntry->rows.int4s )
    {
      ABORT( status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC );
    }
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.int4s) = 0;

    prev = dbEntry;

    /* end time nanoseconds */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    if ( ! dbEntry )
    {
      ABORT( status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC );
    }
    dbEntry->previous = prev;;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "end_time_ns" );

    dbEntry->type       = int_4s;
    dbEntry->numberRows = 1;
    dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
    if ( ! dbEntry->rows.int4s )
    {
      ABORT( status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC );
    }
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.int4s) = 0;

    prev = dbEntry;

    /* amplitude (dummy value) */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    if ( ! dbEntry )
    {
      ABORT( status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC );
    }
    dbEntry->previous = prev;;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "amplitude" );

    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    if ( ! dbEntry->rows.real4 )
    {
      ABORT( status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC );
    }
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = 0.0;

    dbEntry->next = NULL;

  }


  RETURN (status);
}

void
BuildOptionalOutPut (
    LALStatus          *status,
    LALSearchOutput    *output,
    REAL4Vector        *rhosqVec
                    )
{
  outPut               *outputArray;
  multiDimData         *mddEntry;


  INITSTATUS (status, "BuildOptionalOutPut", BUILDOUTPUTC);


  /*
   *
   * create memory for the outPut array
   *
   */


  outputArray = output->result = (outPut *)
    LALCalloc( 1, sizeof(outPut) );
  if ( ! outputArray )
  {
    ABORT( status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC );
  }

  output->numOutput = 1;


  /*
   *
   * fill the array of output
   *
   */


  /* this is the same for every output */
  outputArray->indexNumber = 0;
  outputArray->search         = binaryInspiral;
  outputArray->significant    = 1;
  outputArray->states         = NULL;
  outputArray->results        = NULL;


  /* 
   *
   * alloc memory for the optional linked list
   *
   */

  
  /* allocate memory */
  mddEntry = outputArray->optional = 
    (multiDimData *) LALCalloc( 1, sizeof(multiDimData) );
  if ( ! mddEntry )
  {
    ABORT( status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC );
  }

  /* write the name and domain of the data */
  sprintf( mddEntry->name, "filter_output" );
  sprintf( mddEntry->units, "dimensionless" );
  mddEntry->space = timeD;

  /* data type is REAL4 */
  mddEntry->type = real_4;
  
  /* set the interval union to zero */
  memset( &(mddEntry->range.dTime), 0, sizeof(gpsTimeInterval) );

  /* size of the data */
  mddEntry->numberDimensions = 1;
  mddEntry->dimensions       = (UINT4 *) LALMalloc( sizeof(UINT4) );
  if ( ! mddEntry->dimensions )
  {
    ABORT( status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC );
  }
  mddEntry->dimensions[0]    = rhosqVec->length;

  /* dcHistory is meaningless here */
  mddEntry->history = NULL;

  /* allocate memory for the data */
  mddEntry->data.real4 = (REAL4 *) 
    LALCalloc( rhosqVec->length, sizeof(REAL4) );
  if ( ! mddEntry->data.real4 )
  {
    ABORT( status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC );
  }

  /* and copy the data there */
  memcpy( mddEntry->data.real4, rhosqVec->data, 
      rhosqVec->length * sizeof(REAL4) );

  /* normal exit */
  RETURN (status);
}
