/******** <lalVerbatim file="StackSlideHV"> ********
Author: Mendell, G. and Landry M.
$Id: StackSlide.h,v 1.1 2003/09/25 19:41:58 jolien Exp $
********* </lalVerbatim> ********/

/* Notes: */
/* BLK = one Block of frequency domain data */
/* STK = one Stack of frequency domain data; Block are coherenty combined to make Stacks. */
/* SUM = Summed Stacks after Sliding */

/* ??? Code first submitted to lalwrapper cvs. */

/* REVISIONS: */

#ifndef _STACKSLIDE_H
#define _STACKSLIDE_H

#include <lal/LALRCSID.h>
#include <LALWrapperInterface.h>

NRCSID( STACKSLIDEH, "$Id: StackSlide.h,v 1.1 2003/09/25 19:41:58 jolien Exp $");

/* 10/10/01 gam */
/* 10/11/01 gam */
/* 12/05/01 gam */
/*  10/10/01 gam */
/*  10/11/01 gam */
/* 12/05/01 gam */

/* <lalErrTable> */
#define STACKSLIDEH_ENULL 1
#define STACKSLIDEH_EGPSTINT 2
#define STACKSLIDEH_EDELTAT 3
#define STACKSLIDEH_EINPUTN 4
#define STACKSLIDEH_ETBLK 5
#define STACKSLIDEH_ERA 6
#define STACKSLIDEH_EDEC 7
#define STACKSLIDEH_EFREQ 8
#define STACKSLIDEH_EFREQDERIV 9
#define STACKSLIDEH_EALOC 10
#define STACKSLIDEH_ENODATA 11
#define STACKSLIDEH_ENDATA 12
#define STACKSLIDEH_ETIMESTEP 13
#define STACKSLIDEH_ESTARTTIME 14
#define STACKSLIDEH_ESTOPTIME 15
#define STACKSLIDEH_ENBLK 16
#define STACKSLIDEH_ENTBLK 17
#define STACKSLIDEH_EMCOHBLKPERCALL 18
#define STACKSLIDEH_EIFONICKNAME 19
#define STACKSLIDEH_EIFO 20
#define STACKSLIDEH_ETARGETNAME 21
#define STACKSLIDEH_ECHANNEL 22
#define STACKSLIDEH_EISTARTTIME 23
#define STACKSLIDEH_EMISSINGBLKDATA 24
#define STACKSLIDEH_ESTARTFREQ 25
#define STACKSLIDEH_EFREQSTEPSIZE 26

#define STACKSLIDEH_MSGENULL           "Null pointer"
#define STACKSLIDEH_MSGEGPSTINT        "Unexpected GPS time interval"
#define STACKSLIDEH_MSGEDELTAT         "Invalid deltaT"
#define STACKSLIDEH_MSGEINPUTN         "Unexpected number of input data points"
#define STACKSLIDEH_MSGETBLK           "Invalid tBLK"
#define STACKSLIDEH_MSGERA             "Invalid right ascension"
#define STACKSLIDEH_MSGEDEC            "Invalid declination"
#define STACKSLIDEH_MSGEFREQ           "Invalid frequency +/- 0.5*band (could be negative, outside LIGO band, or too high for sample rate)"
#define STACKSLIDEH_MSGEFREQDERIV      "One of the frequency derivatives is possibly too large; frequency will evolve outside allowed band"
#define STACKSLIDEH_MSGEALOC           "Memory allocation error"
#define STACKSLIDEH_MSGENODATA         "No input data was found"
#define STACKSLIDEH_MSGENDATA          "Invalid number of input data points"
#define STACKSLIDEH_MSGETIMESTEP       "Incorrect input data time step"
#define STACKSLIDEH_MSGESTARTTIME      "Incorrect input data start time"
#define STACKSLIDEH_MSGESTOPTIME       "Incorrect input data stop time"
#define STACKSLIDEH_MSGENBLK           "Invalid nSamplesPerBLK"
#define STACKSLIDEH_MSGENTBLK          "nSamplesPerBLK and tBLK are inconsistent with deltaT"
#define STACKSLIDEH_MSGEMCOHBLKPERCALL "Invalid numSUMsPerCall"
#define STACKSLIDEH_MSGEIFONICKNAME    "Invalid or null ifoNickName"
#define STACKSLIDEH_MSGEIFO            "Invalid or null IFO"
#define STACKSLIDEH_MSGETARGETNAME     "Invalid or null Target Name"
#define STACKSLIDEH_MSGECHANNEL        "Invalid or null tDomainChannel"
#define STACKSLIDEH_MSGEISTARTTIME     "Requested GPS start time resulted in invalid index to input data."
#define STACKSLIDEH_MSGEMISSINGBLKDATA "Some requested input BLK data is missing"
#define STACKSLIDEH_MSGESTARTFREQ      "Input BLK start frequency does not agree with that requested"
#define STACKSLIDEH_MSGEFREQSTEPSIZE   "Input BLK frequency step size does not agree with that expected"

/* </lalErrTable> */

/*  Define the LIGO BAND */ /* Changed 11/10/01 gam */
#define STACKSLIDEH_LIGOBANDMIN 0.0
#define STACKSLIDEH_LIGOBANDMAX 2048.0

/*
 * The user-defined structure which holds data relevant to this particular
 * wrapper shared object
*/

/* Define structure to hold statistics about a power series */
typedef struct
tagFreqSeriesPowerStats
{
	REAL8 pwMax;      /* max power */
	REAL8 freqPWMax;  /* frequency associated with pwMax */
	REAL8 pwMean;     /* mean power */
	REAL8 pwStdDev;   /* std dev of power */
}
FreqSeriesPowerStats;

typedef struct
tagStackSlideSearchParams
{

  /******************************************/
  /*                                        */
  /* START SECTION: parameters passed as    */
  /* arguments.  Those indented are         */
  /* computed from these.                   */
  /*                                        */
  /******************************************/

  UINT4   gpsStartTimeSec;           /* GPS start time of data requested seconds */
  UINT4   gpsStartTimeNan;           /* GPS start time of data requested nanoseconds */
  REAL8   sampleRate;                /* Sample rate of the time-domain data used to make the frequency-domain input data Blocks */
    REAL8   deltaT;                    /* Time step size in seconds = 1.0/sampleRate */
  REAL8   duration;                  /* Total time being analyzed  */
    UINT4   totalNumTimeSamples;       /* Total Number of time samples = sampleRate*duration */

  INT4    numBLKs;                   /* Number of input BLKs.  Not duration/tBLK if gaps are present */
  REAL8   tBLK;                      /* duration in seconds of BLKs */
    INT4    numBLKsPerDuration;        /* Equals duration/tBLK; but not acual number of blocks if gaps exist in the data */
    INT4    nSamplesPerBLK;            /* Equals sampleRate*tBLK = number of time domain data points used to compute one BLKs */
  REAL8   tEffBLK;                   /* Effective duration of BLK such that dfBLK = 1.0/tEffBLK  */
    REAL8   dfBLK;                     /* Freq resolution of BLK = 1.0/tEffBLK  Could be different from 1/tBLK due to oversampling  */
  REAL8   f0BLK;                     /* Start frequency of the input BLKs */
  REAL8   bandBLK;                   /* Band width of the input BLKs. */
  INT4    nBinsPerBLK;               /* Number of data points in the input BLK in the input frequency band */

  INT4    numBLKsPerSTK;             /* Number of BLKs to use to make one STK */
    INT4    numSTKs;                   /* Number of actual STKs. Not duration/tSTK if gaps are present */
    REAL8   tSTK;                      /* duration in seconds of STKs = tBLK*numBLKsPerSTK */
  REAL8   tEffSTK;                   /* Effective duration of STK such that dfSTK = 1.0/tEffSTK */
    REAL8   dfSTK;                     /* Freq resolution of STK = 1.0/tEffSTK. Could be different from 1/tSTK due to oversampling  */
  REAL8   f0STK;                     /* Start frequency of STKs */
  REAL8   bandSTK;                   /* Band width of the STKs. */
  INT4    nBinsPerSTK;               /* Number of data points in the STKs in the STK frequency band */

  INT4    numSTKsPerSUM;             /* Number of STKs to use to make one SUM (Usually duration/tSTK) */
    INT4    numSUMsPerParamSpacePt;  /* Number of output SUMs per parameter space point = params->duration/params->tSUM. (Usually will = 1) */
    REAL8   tSUM;                      /* duration in seconds of output SUMs = tSTK*numSTKsPerSUM. (Usually = duration) */
  REAL8   tEffSUM;                   /* Effective duration of SUM such that dfSUM = 1.0/tEffSUM */
    REAL8   dfSUM;                     /* Freq resolution of output SUMs.  dfSUM = 1.0/tEffSUM. Could be different from dfSTK in some algorithms?  */
  REAL8   f0SUM;                     /* Start frequency of output SUMs */
  REAL8   bandSUM;                   /* Band width of the output SUMs. */
  INT4    nBinsPerSUM;               /* Number of data points in the output SUMs in the input frequency band */

  CHAR    *ifoNickName;              /* 2 character ifoNickName = H1, H2, or L1; note IFO is the site = LHO, LLO, GEO, etc.... */
  CHAR    *IFO;                      /* Identifies the interferometer site = LHO, LLO, GEO, etc.... */
  CHAR    *patchName;                /* Name of the patch in parameter space of the search (e.g, Galactic Center) */
  INT4    patchNumber;               /* Index of the patch in parameter space of the search (will be -1 until indexing implemented) */

  CHAR    tDomainChannel[dbNameLimit];  /* Time domain channel name used to produce blocks */
  CHAR    fDomainChannel[dbNameLimit];  /* Frequency domain channel name that blocks are stored as */

  INT2    inputDataTypeFlag;         /* 0 = SFTs, 1 = PSDs, 2 = F-stat */

  INT2    parameterSpaceFlag;        /* 0 = use input delta for each param, 1 = params are input as vectors of data, 2 = use input param metric, 3 = create param metric */
    INT4    numParamSpacePts;          /* Total number of points in the parameter space to cover */
    INT4    numSUMsTotal;              /* Total Number of Sums output = numSUMsPerParamPt*numParamSpacePts */

  INT2    stackTypeFlag;             /* 0 means stacks are PSDs from SFTs, 1 means stacks are F-statistic from SFTs */
  INT4    Dterms;                    /* = terms used in the computation of the dirichlet kernel in LALDemod (Default value is 32) */

  INT2    thresholdFlag;             /* How to apply the threshold; 0 = if power above threshold 1 put result in database */
  REAL8   threshold1;                /* Threshold 1 for put results in the database (could be on power or false alarm rate) */
  REAL8   threshold2;                /* Threshold 1 for put results in the database (could be on power or false dismisal rate) */

  INT2    calibrationFlag;           /* Flag that specifies what calibration to do; -1 means Blks are already calibrated, 0 means leave uncalibrated; 1 calibrate  */

  INT2    weightFlag;                /* Flag that specifies whether to weight BLKs or STKs with a(t) or b(t).  */

  INT2    windowFilterFlag;          /* Flag that specifies whether any filtering or windowing was done or should be done. (If < 0 then specifies what was done in the time domain) */
  REAL8   windowFilterParam1;        /* 1st parameter to use in windowing or filtering */
  REAL8   windowFilterParam2;        /* 2nd paramter to use in windowing or filtering */
  REAL8   windowFilterParam3;        /* 3rd paramter to use in windowing or filtering */

  INT2    normalizationFlag;         /* Flag that specifies what normalization to do.  If < 0 then specifies what normalization was already done. */
  REAL8   f0NRM;                     /* Start frequency to normalize over to create NRMs */
  REAL8   bandNRM;                   /* Band width to normalize over to create NRMs. */
  INT4    nBinsPerNRM;               /* Number of data points to normalize over to create to creat NRMs */

  INT2    testFlag;                  /* Specifies any tests or debugging to do; 0 means no test */

  UINT4   numSUMsPerCall;            /* If > 0 then = number of SUMs (parameter space points) to compute during each call to ApplySearch */
  INT2    outputFlag;                /* Flag that specifies what to output; if > 0 then will output Sums into frame file */

  INT2    unusedFlag1;               /* Place holder for future flag */
  REAL8   unusedParam1;              /* Place holder for future parameter */
  INT2    unusedFlag2;               /* Place holder for future flag */
  REAL8   unusedParam2;              /* Place holder for future parameter */

  REAL8   startRA;                   /* Starting Right Ascension in radians */
  REAL8   stopRA;                    /* Ending Right Ascension in radians */
  REAL8   deltaRA;                   /* Right Ascension step size in radians */
  INT4    numRA;                     /* Number of RA to compute SUMs for */

  REAL8   startDec;                  /* Starting Declination in radians */
  REAL8   stopDec;                    /* Ending Declination in radians  */
  REAL8   deltaDec;                   /* Declination step size in radians */
  INT4    numDec;                     /* Number of Dec to compute SUMs for */

  INT4    numSpinDown;               /* Number of nonzero spindown parameters */

  REAL8   startFDeriv1;              /* Starting 1st deriv of freq in Hz/s */
  REAL8   stopFDeriv1;               /* Ending 1st deriv of freq in Hz/s */
  REAL8   deltaFDeriv1;              /* 1st deriv of freq step size in Hz/s */
  INT4    numFDeriv1;                /* Number of 1st derivs to compute SUMs for */

  REAL8   startFDeriv2;              /* Starting 2nd deriv of freq in Hz/s^2 */
  REAL8   stopFDeriv2;               /* Ending 2nd deriv of freq in Hz/s^2 */
  REAL8   deltaFDeriv2;              /* 2nd deriv of freq step size in Hz/s^2 */
  INT4    numFDeriv2;                /* Number of 2nd derivs to compute SUMs for */

  REAL8   startFDeriv3;              /* Starting 3rd deriv of freq in Hz/s^3 */
  REAL8   stopFDeriv3;               /* Ending 3rd deriv of freq in Hz/s^3 */
  REAL8   deltaFDeriv3;              /* 3rd deriv of freq step size in Hz/s^3 */
  INT4    numFDeriv3;                /* Number of 3rd derivs to compute SUMs for */

  REAL8   startFDeriv4;              /* Starting 4th deriv of freq in Hz/s^4 */
  REAL8   stopFDeriv4;               /* Ending 4th deriv of freq in Hz/s^4 */
  REAL8   deltaFDeriv4;              /* 4th deriv of freq step size in Hz/s^4 */
  INT4    numFDeriv4;                /* Number of 4th derivs to compute SUMs for */

  REAL8   startFDeriv5;              /* Starting 5th deriv of freq in Hz/s^5 */
  REAL8   stopFDeriv5;               /* Ending 5th deriv of freq in Hz/s^5 */
  REAL8   deltaFDeriv5;              /* 5th deriv of freq step size in Hz/s^5 */
  INT4    numFDeriv5;                /* Number of 5th derivs to compute SUMs for */

  /******************************************/
  /*                                        */
  /* END SECTION: parameters passed as      */
  /* arguments.  Those indented are         */
  /* computed from these.                   */
  /*                                        */
  /******************************************/

  /******************************************/
  /*                                        */
  /* START SECTION: other parameters        */
  /*                                        */
  /******************************************/

  CHAR  *dsoName;         /* Name of this DSO */ /* 11/05/01 gam */

  /* Basic beowulf node descriptors */
  BOOLEAN searchMaster;           /* TRUE on the search master */
  UINT4   rank;                   /* Rank of this slave */
  UINT4   numSlaves;              /* Total number of slaves */
  UINT4   curSlaves;              /* Current number of slaves still working */
  UINT4   numNodes;               /*  07/12/02 gam  Should equal number of slaves + 1 for search master */

  UINT4  numDBOutput;             /* Used to keep track of the number of rows written to the database */

  gpsTimeInterval times;          /* The GPS start and end time of the data */

  LALUnit     unitBLK;            /* Unit BLKs are stored in (e.g., strain per root Hz) */

  FFT **BLKData;                   /* Container for BLKs */
  REAL4FrequencySeries **STKData;  /* Container for STKs */
  REAL4FrequencySeries **SUMData;  /* Container for SUMs */

  INT4 iMinBLK;      /* Index of minimum frequency in BLK band */
  INT4 iMaxBLK;      /* Index of maximum frequency in BLK band */
  INT4 iMinNRM;      /* Index of mimimum frequency to include when normalizing BLKs */
  INT4 iMaxNRM;      /* Index of maximum frequency to include when normalizing BLKs */

  LIGOTimeGPS *timeStamps;          /* Container for time stamps giving initial time for which each BLK was computed. */
  gpsTimeInterval *timeIntervals;   /* Array of time intervals; needed for building frame output */

  EphemerisData *edat; /* 07/10/02 gam Add EphemerisData *edat = pointer to ephemeris data to StackSlideSearchParams */

  UINT4 firstK;           /* Index used to keep track of which BLK, STK, or SUM to work on */
  UINT4 lastK;            /* Index used to keep track of which BLK, STK, or SUM to work on */
  BOOLEAN finishedBLKs;   /* Set equal to true when all BLKS for this job have been found in input data */
  BOOLEAN finishedSTKs;   /* Set equal to true when all STKs for this job have been created */
  BOOLEAN finishedSUMs;   /* Set equal to true when all BLKS for this job have been created */

  INT4 whichSTK;          /* which STK does BLK go with  */
  INT4 lastWhichSTK;      /* Last value of whichSTK does BLK go with  */

  REAL8 **skyPosData;     /* Container for Parameter Space Data */
  REAL8 **freqDerivData;  /* Container for Frequency Derivative Data */
  INT4 numSkyPosTotal;    /* Total number of Sky positions to cover */
  INT4 numFreqDerivTotal; /* Total number of Frequency evolution models to cover */

  FreqSeriesPowerStats *SUMStats; /* Container for statistics about each SUM */

  /******************************************/
  /*                                        */
  /* END SECTION: other parameters          */
  /*                                        */
  /******************************************/


}
StackSlideSearchParams;

#endif /* _STACKSLIDE_H */
