/********************************** <lalVerbatim file="PowerInitSearchCV">
Author: Torres, C
$Id: BuildOutput.c,v 1.2 2003/10/08 17:02:22 jolien Exp $
**************************************************** </lalVerbatim> */

#include <lal/LALStdlib.h>
#include <lal/LALDatatypes.h>
#include <lal/ExcessPower.h>
#include <lal/TFTransform.h>
#include <lal/BurstSearch.h>
#include <lal/Comm.h>
#include <lal/AVFactories.h>
#include <lal/EPData.h>
#include <lal/LALConstants.h>
#include "TrackSearchApplySearch.h"


/* #define POWER_SO_ATTACH_DDD */

NRCSID( BUILDPOWEROUTPUTC, "power $Id: BuildOutput.c,v 1.2 2003/10/08 17:02:22 jolien Exp $");
 
void
BuildTrackSearchOutput (
    LALStatus               *status,
    LALSearchOutput         *output,
    TrackSearchEventList    *TSEventList,
    TSSearchParams          *params
                    )
{
  TrackSearchEvent     *event;               /* dummy pointer */
  outPut               *outputArray;         /* dummy pointer */
  dataBase             *prev;                /* dummy pointer */
  dataBase             *dbEntry;             /* dummy pointer */
  INT4                  i;                   /* general index */
  TrackSearchEvent     *curveEvent=NULL;

  INITSTATUS (status, "BuildPowerOutPut", BUILDPOWEROUTPUTC);

  output->numOutput = 1;
  event = TSEventList->eventlist;

  /*
   *
   * print the list events as recieved by the master
   *
   */


#ifdef POWER_SO_DBGLVL3
  {
    INT4 j=0;
    event = TSEventList->eventlist;

    do
    {

      fprintf( stdout, "build output: numEvents %d\n", params->numEvents);
      fflush( stdout );
      fprintf( stdout, "build output: confidence = %e\n", event->confidence );
      fflush( stdout );

      event = event->nextEvent;
      j++;

    }while( j<params->numEvents);
  }
#endif


  /*
   *
   * create memory for the outPut array
   *
   */

  outputArray = output->result = (outPut *) LALMalloc( sizeof(outPut) );


  /*
   *
   * fill the array of output
   *
   */

  (*outputArray).search         = burst;
  (*outputArray).significant    = 1;
  (*outputArray).states         = NULL;
  (*outputArray).optional       = NULL;
  (*outputArray).results        = NULL;

  /* May not be correct?  Does this need to be non-zero */
  (*outputArray).indexNumber = 0;


  /* 
   *
   * alloc memory for the database linked list
   *
   */

  { 
    INT4 numEvents = TSEventList->numEventMaps;

    /* entry: start_time */
    dbEntry = (*outputArray).results = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = NULL;
    sprintf( dbEntry->tableName, "TrackSearch Curve" );
    sprintf( dbEntry->columnName, "start_time" );
    dbEntry->type       = int_4s;
    dbEntry->numberRows = numEvents;
    dbEntry->rows.int4s = (INT4 *) LALMalloc( numEvents*sizeof(INT4) );
    dbEntry->rowDimensions = NULL;
    for ( i = 0, event = curveEvent; (i < TSEventList->numEventMaps) && event; 
        ++i, event = event->nextEvent )
    {
      (dbEntry->rows.int4s)[i] = event->mapStartTime.gpsSeconds;
    }

    prev = dbEntry; 

    /* entry: start time nanosecond component*/
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
    sprintf( dbEntry->tableName, "TrackSearch Curve" );
    sprintf( dbEntry->columnName, "start_time_ns" );
    dbEntry->type       = int_4s;
    dbEntry->numberRows = numEvents;
    dbEntry->rows.int4s = (INT4 *) LALMalloc( numEvents*sizeof(INT4) );
    dbEntry->rowDimensions = NULL;
    for ( i = 0, event = curveEvent; (i < TSEventList->numEventMaps) && event; 
        ++i, event = event->nextEvent )
    {
      (dbEntry->rows.int4s)[i] = event->mapStartTime.gpsNanoSeconds;
    }

    prev = dbEntry;

    /* entry: duration in sampling points*/
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
    sprintf( dbEntry->tableName, "TrackSearch Curve" );
    sprintf( dbEntry->columnName, "duration" );
    dbEntry->type       = int_4s;
    dbEntry->numberRows = numEvents;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( numEvents*sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    for ( i = 0, event = curveEvent; (i < TSEventList->numEventMaps) && event; 
        ++i, event = event->nextEvent )
    {
      (dbEntry->rows.real4)[i] = params->TimeLengthPoints;
    }

    prev = dbEntry;

    
    /* entry: start frequency pixel */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
    sprintf( dbEntry->tableName, "TrackSearch Curve" );
    sprintf( dbEntry->columnName, "start_freq_pixel" );
    dbEntry->type       = int_4s;
    dbEntry->numberRows = numEvents;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( numEvents*sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    for ( i = 0, event = curveEvent; (i < TSEventList->numEventMaps) && event; 
        ++i, event = event->nextEvent )
    {
      (dbEntry->rows.real4)[i] = event->FstartPixel;
    }

    prev = dbEntry; 

    /* entry: start time pixel */
 dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
    sprintf( dbEntry->tableName, "TrackSearch Curve" );
    sprintf( dbEntry->columnName, "start_time_pixel" );
    dbEntry->type       = real_4;
    dbEntry->numberRows = numEvents;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( numEvents*sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    for ( i = 0, event = curveEvent; (i < TSEventList->numEventMaps) && event; 
        ++i, event = event->nextEvent )
    {
      (dbEntry->rows.real4)[i] = event->TstartPixel;
    }

    prev = dbEntry; 

    /* entry: peakPixel frequency */
 dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
    sprintf( dbEntry->tableName, "TrackSearch Curve" );
    sprintf( dbEntry->columnName, "peak_freq_pixel" );
    dbEntry->type       = real_4;
    dbEntry->numberRows = numEvents;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( numEvents*sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    for ( i = 0, event = curveEvent; (i < TSEventList->numEventMaps) && event; 
        ++i, event = event->nextEvent )
    {
      (dbEntry->rows.real4)[i] = event->peakPixelF;
    }

    prev = dbEntry; 

    /* entry: peakPixel Time */
 dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
    sprintf( dbEntry->tableName, "TrackSearch Curve" );
    sprintf( dbEntry->columnName, "peak_time_pixel" );
    dbEntry->type       = real_4;
    dbEntry->numberRows = numEvents;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( numEvents*sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    for ( i = 0, event = curveEvent; (i < TSEventList->numEventMaps) && event; 
        ++i, event = event->nextEvent )
    {
      (dbEntry->rows.real4)[i] = event->peakPixelT;
    }

    prev = dbEntry; 

    /* entry: peakPixel Frequency */
 dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
    sprintf( dbEntry->tableName, "TrackSearch Curve" );
    sprintf( dbEntry->columnName, "peak_frequency_pixel" );
    dbEntry->type       = real_4;
    dbEntry->numberRows = numEvents;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( numEvents*sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    for ( i = 0, event = curveEvent; (i < TSEventList->numEventMaps) && event; 
        ++i, event = event->nextEvent )
    {
      (dbEntry->rows.real4)[i] = event->peakPixelF;
    }

    prev = dbEntry; 

    /* entry: end pixel time */
 dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
    sprintf( dbEntry->tableName, "TrackSearch Curve" );
    sprintf( dbEntry->columnName, "stop_time_pixel" );
    dbEntry->type       = real_4;
    dbEntry->numberRows = numEvents;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( numEvents*sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    for ( i = 0, event = curveEvent; (i < TSEventList->numEventMaps) && event; 
        ++i, event = event->nextEvent )
    {
      (dbEntry->rows.real4)[i] = event->TstopPixel;
    }

    prev = dbEntry; 

    /* entry: end pixel frequency */
 dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
    sprintf( dbEntry->tableName, "TrackSearch Curve" );
    sprintf( dbEntry->columnName, "stop_frequency_pixel" );
    dbEntry->type       = real_4;
    dbEntry->numberRows = numEvents;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( numEvents*sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    for ( i = 0, event = curveEvent; (i < TSEventList->numEventMaps) && event; 
        ++i, event = event->nextEvent )
    {
      (dbEntry->rows.real4)[i] = event->FstopPixel;
    }

    prev = dbEntry; 

    /* entry: curve power */
 dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
    sprintf( dbEntry->tableName, "TrackSearch Curve" );
    sprintf( dbEntry->columnName, "curve_power" );
    dbEntry->type       = real_4;
    dbEntry->numberRows = numEvents;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( numEvents*sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    for ( i = 0, event = curveEvent; (i < TSEventList->numEventMaps) && event; 
        ++i, event = event->nextEvent )
    {
      (dbEntry->rows.real4)[i] = event->power;
    }

    prev = dbEntry; 

   /* entry: presence of junction */
 dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
    sprintf( dbEntry->tableName, "TrackSearch Curve" );
    sprintf( dbEntry->columnName, "curve_junction" );
    dbEntry->type       = real_4;
    dbEntry->numberRows = numEvents;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( numEvents*sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    for ( i = 0, event = curveEvent; (i < TSEventList->numEventMaps) && event; 
        ++i, event = event->nextEvent )
    {
      (dbEntry->rows.real4)[i] = event->junction;
    }

    prev = dbEntry; 


    dbEntry->next = NULL;

  }

  RETURN (status);
}
