/*----------------------------------------------------------------------- 
 * 
 * File Name: TSExch.c
 *
 * Author: Charlie Torres
 * 
 * Revision: $Id: TSExch.c,v 1.1 2003/10/08 15:05:12 jolien Exp $
 * 
 *-----------------------------------------------------------------------
 */

#include <lal/AVFactories.h>
#include <lal/Comm.h>
#include <lal/ExcessPower.h>/*?*/
#include <lal/BurstSearch.h>/*?*/
#include "TSExch.h"
#include <lal/TrackSearch.h>



NRCSID (EPEXCHC, "$Id: TSExch.c,v 1.1 2003/10/08 15:05:12 jolien Exp $");

void
LALExchangeTSEventList (
    LALStatus              *status,
    TrackSearchEvent      **eventHead,
    INT4                    numEvents,
    ExchParams             *exchParams
    )
{
  INT4      i, code;
  TrackSearchEvent   *eventCurrent = NULL;
  TrackSearchEvent   *prev         = NULL;
  TrackSearchEvent   *events        = NULL;
  MPI_Status stat;

  INITSTATUS (status, "LALExchangeTSEventList", EPEXCHC);
  ATTATCHSTATUSPTR (status);

  if (exchParams->send)  /* I am sending */
  {

    /* check that we have events to send */
    ASSERT (*eventHead, status, TSEXCH_ENULL, TSEXCH_MSGENULL);

    /* allocate memory for array of pointers to events */
    events = NULL;
    events = (TrackSearchEvent *) LALMalloc (numEvents * sizeof(TrackSearchEvent));

    /*  Make sure that the allocation was succesful */
    if ( !(events) ){
      ABORT (status, TSEXCH_ENULL, TSEXCH_MSGENULL);
    }

    /* copy out events into array */
    i=0;
    eventCurrent = *eventHead;
    while (eventCurrent != NULL && i < numEvents)
    {
      i++;
      memcpy( (events + i-1), eventCurrent, sizeof(TrackSearchEvent));
      eventCurrent = eventCurrent->nextEvent;
    }

    code = MPI_Send( events, numEvents * sizeof(TrackSearchEvent), MPI_BYTE, 
        exchParams->partnerProcNum, MPIZ, exchParams->mpiComm );
    if ( code != MPI_SUCCESS )
    {
      ABORT( status, TSEXCH_EMPIE, TSEXCH_MSGEMPIE);
    }

    LALFree(events);

  }
  else /* I am receiving */
  {

    /* check that this is a new list */
    ASSERT (!*eventHead, status, TSEXCH_ENNUL, TSEXCH_MSGENNUL);

    /* allocate memory for array of pointers to events */
    events = NULL;
    events = (TrackSearchEvent *) LALMalloc (numEvents * sizeof(TrackSearchEvent));

    /*  Make sure that the allocation was succesful */
    if ( !(events) ){
      ABORT (status, TSEXCH_ENULL, TSEXCH_MSGENULL);
    }

    code = MPI_Recv( events, numEvents * sizeof(TrackSearchEvent), MPI_BYTE, 
        exchParams->partnerProcNum, MPIZ, exchParams->mpiComm, &stat );
    if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
    {
      ABORT( status, TSEXCH_EMPIE, TSEXCH_MSGEMPIE);
    }

    i=0;
    /* Parse the events into a linked list*/
    do
    {
      /* create memory for the first event */
      eventCurrent = (TrackSearchEvent *) LALMalloc (sizeof(TrackSearchEvent));
      ASSERT (eventCurrent, status, TSEXCH_ENULL, TSEXCH_MSGENULL);

      /* make a note of the first node in the list to return */
      if ( *eventHead == NULL ) *eventHead = eventCurrent;

      /* copy the event from the array */
      memcpy( eventCurrent, (events + i), sizeof(TrackSearchEvent));
      
      /* point the previous node to this node */
      if ( prev != NULL ) prev->nextEvent = eventCurrent;
      prev = eventCurrent;

      /* increment pointer */
      ++i;
    } while ( eventCurrent->nextEvent != NULL && i<numEvents);

    /* Terminate the linked list */
    eventCurrent->nextEvent = NULL;

    LALFree(events);

  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}


